/* db_mdio.c - serial i/o routines
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include "windows.h"

#include <stdio.h>
#include <stdarg.h>
#include "db_xmodem_v010.h"

static HANDLE hSerial;
static COMMTIMEOUTS timeouts;

int md_init(char *port)
{
    DCB state;
    
    hSerial = CreateFile(
        port,
        GENERIC_READ | GENERIC_WRITE,
        0,
        NULL,
        OPEN_EXISTING,
        0,
        NULL);

    if (hSerial == INVALID_HANDLE_VALUE)
        return FALSE;

    GetCommState(hSerial, &state);
    state.BaudRate = CBR_9600;
    state.fBinary = TRUE;
    state.fParity = FALSE;
    state.fOutxCtsFlow = FALSE;
    state.fOutxDsrFlow = FALSE;
    state.fDtrControl = DTR_CONTROL_DISABLE;
    state.fDsrSensitivity = FALSE;
    state.fTXContinueOnXoff = TRUE;
    state.fOutX = FALSE;
    state.fInX = FALSE;
    state.fErrorChar = FALSE;
    state.fNull = FALSE;
    state.fRtsControl = RTS_CONTROL_DISABLE;
    state.fAbortOnError = FALSE;
    state.ByteSize = 8;
    state.Parity = NOPARITY;
    state.StopBits = ONESTOPBIT;
    SetCommState(hSerial, &state);

    GetCommTimeouts(hSerial, &timeouts);
    timeouts.ReadIntervalTimeout = MAXDWORD;
    timeouts.ReadTotalTimeoutMultiplier = MAXDWORD;

    return TRUE;
}

void md_done(void)
{
    CloseHandle(hSerial);
}

int md_write(char *buf, int len)
{
    DWORD written;
    BOOL ok;
	fprintf(stderr, "md_write: %02x %02x\n", buf[0], buf[1]);
    ok = WriteFile(hSerial, (void *)buf, (DWORD)len, &written, NULL);
    return ok && (int)written == len;
}

int md_put(int ch)
{
    char buf[1];
    buf[0] = ch;
    return md_write(buf, 1);
}

int md_get(int timeout)
{
    unsigned char buf[1];
    DWORD read;
    BOOL ok;
    timeouts.ReadTotalTimeoutConstant = timeout * 1000;
    SetCommTimeouts(hSerial, &timeouts);
    ok = ReadFile(hSerial, (void *)buf, (DWORD)1, &read, NULL);
    return ok && read == 1 ? buf[0] : DT_TIME;
}

void md_iflush(void)
{
    while (md_get(1) != DT_TIME)
        ;
}

void md_log(char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    fputs("mdio: ", stderr);
    vfprintf(stderr, fmt, ap);
    putc('\n', stderr);
    va_end(ap);
}

