/* xmodem.h - x/ymodem transfer routine definitions */
/*
	by David Betz, BYTE Magazine/BIX
*/

#ifndef __DB_XMODEM_H__
#define __DB_XMODEM_H__

/* useful definitions */
#ifndef TRUE
#define TRUE    1
#define FALSE   0
#endif

/* block lengths */
#define SBLKLEN		128
#define LBLKLEN		1024

/* error codes */
#define ER_OK		0
#define ER_INIT		1
#define ER_DATA		2
#define ER_EOT		3
#define ER_CANCEL	4
#define ER_EOF		5
#define ER_DONE		6
#define ER_HEAD		7

/* protocol characters */
#define SOH     0x01    /* start of a 128 byte block */
#define STX     0x02    /* start of a 1024 byte block */
#define EOT     0x04    /* end of a complete file */
#define ACK     0x06    /* positive acknowledgement */
#define NAK     0x15    /* negative acknowledgement */
#define CAN     0x18    /* cancel transfer */
#define CRC     'C'     /* CRC request (instead of NAK to start a transfer) */

/* useful constants */
#define DT_TIME -1      /* return value for timeout */
#define RETRY   10      /* number of times to retry */

/* timeout values (in seconds) */
#define STIME   6       /* timeout between characters of a packet */
#define ITIME   5       /* timeout sending initial NAK or 'C' */
#define XTIME   60      /* timeout waiting for initial NAK */
#define LTIME   20      /* timeout waiting for an ACK/NAK */

/* crc table */
extern unsigned short crctab[];

/*
 * updcrc macro derived from article Copyright (C) 1986 Stephen Satchell. 
 *  NOTE: First argument is referenced twice.
 *        Second argument must be in range 0 to 255.
 * 
 * Programmers may incorporate any or all code into their programs, 
 * giving proper credit within the source. Publication of the 
 * source routines is permitted so long as proper credit is given 
 * to Stephen Satchell, Satchell Evaluations and Chuck Forsberg, 
 * Omen Technology.
 */

#define updcrc(crc,ch) (crctab[(((crc) >> 8) & 0xFF)] ^ ((crc) << 8) ^ (ch))

/* prototypes */
int xy_sinit(void);
int xy_sndhdr(char *name, long size);
int xy_sndblk(unsigned char *buf,int len);
int xy_sndeot(void);
int xy_sndend(void);
int xy_rinit(int usecrc);
int xy_rcvhdr(int usecrc, char *name, long *psize);
int xy_rcvblk(unsigned char *buf, int *plen);
void xy_sndack(void);
void xy_cancel(void);

int md_init(char *port);
void md_done(void);
int md_get(int tmo);
void md_iflush(void);
int md_put(int ch);
int md_write(char *buf, int len);
void md_log(char *fmt, ...);

#endif
