/* db_packet_v010.h - packet transfer routine definitions
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#ifndef __DB_PACKET_H__
#define __DB_PACKET_H__

#include "db_types_v010.h"

/* packet format:

  SOH pkt# type length hdrchk length*data crc1 crc2

*/

/* packet types */
#define PKT_DEBUG	0x00
#define PKT_ACK		0x01
#define PKT_NAK		0x02
#define PKT_BASE	0x10
#define PKT_OPEN	(PKT_BASE + 0)
#define PKT_DATA	(PKT_BASE + 1)
#define PKT_CLOSE	(PKT_BASE + 2)
#define PKT_RUN     (PKT_BASE + 3)
#define PKT_TIMEOUT	-1

/* block lengths */
#define PKTMAXLEN	128
#define FRAMELEN	(5 + PKTMAXLEN + 2)

/* error codes */
#define ER_OK		0
#define ER_DATA		1

/* protocol characters */
#define SOH     0x01    /* start of a 128 byte block */
#define ACK     0x06    /* positive acknowledgement */
#define NAK     0x15    /* negative acknowledgement */
#define CAN     0x18    /* cancel transfer */

#define RETRY   10      /* number of times to retry */
#define XTIME   60      /* timeout waiting for initial NAK */

/* crc table */
extern unsigned short crctab[];

/*
 * updcrc macro derived from article Copyright (C) 1986 Stephen Satchell. 
 *  NOTE: First argument is referenced twice.
 *        Second argument must be in range 0 to 255.
 * 
 * Programmers may incorporate any or all code into their programs, 
 * giving proper credit within the source. Publication of the 
 * source routines is permitted so long as proper credit is given 
 * to Stephen Satchell, Satchell Evaluations and Chuck Forsberg, 
 * Omen Technology.
 */

#define updcrc(crc,ch) (crctab[(((crc) >> 8) & 0xFF)] ^ ((crc) << 8) ^ (ch))

/* db_sendfile.c */
int SendFile(char *srcname, char *dstname);
int RunFile(char *dstname);

/* db_packet.c */
int PktInit(char *port);
void PktSendFrame(int type, uint8_t *buf, int len);
void PktBuildFrame(int type, uint8_t *buf, int len);
void PktSendCurrentFrame(void);
int PktPoll(int timeout);

/* serial i/o definitions */
#define MD_TIMEOUT	-1

/* serial i/o routines */
int md_init(char *port);
void md_done(void);
int md_get(int timeout);
int md_put(int ch);
int md_write(uint8_t *buf, int len);
void md_log(char *fmt, ...);

#endif
