#include "XGS_PIC_SYSTEM_V010.h"            
#include "XGS_PIC_UART_DRV_V010.h"

#include "db_packet_v010.h"

#define EnableTimerInt()    (IEC0bits.T3IE = 1)
#define DisableTimerInt()   (IEC0bits.T3IE = 0)

static uint16_t timerCount;

int md_init(char *port)
{
    /* start timer 3 running at 10Hz */
    TMR3 = 0;               // clear the current timer value
    PR3 = 19531;            // 50mhz / 256 / 19531 = ~.1 seconds
    T3CONbits.TCKPS = 3;    // 1:256 prescale
    T3CONbits.TCS = 0;      // use internal clock source
    IPC2bits.T3IP = 4;      // set interrupt priority to 4 (0->7)
    IFS0bits.T3IF = 0;      // clear interrupt flag in status register
    T3CONbits.TON = 1;      // turn the timer on

    timerCount = 0;

    EnableTimerInt();
    
    return TRUE;
}

int md_get(int timeout)
{
    int lastTime = timerCount;
    timeout *= 10;
    for (;;) {
        int thisTime;
        uint8_t ch;
        if (UART_getchar(&ch))
            return (int)ch;
        if ((thisTime  = timerCount) - lastTime >= timeout)
            return MD_TIMEOUT;
    }
}

int md_put(int ch)
{
    UART_putchar(ch);
    return TRUE;
}

int md_write(uint8_t *buf, int len)
{
    while (--len >= 0)
        UART_putchar(*buf++);
    return TRUE;
}

/* timer usage:

    1:   video
    2:   sound
    3:   packet
    4-9: unused

*/

void __attribute__((interrupt, no_auto_psv)) _T3Interrupt(void)
{
    ++timerCount;
   IFS0bits.T3IF = 0;       // Clear interrupt flag so it can interrupt again
} 
