/* db_vmimage_v010.c - bytecode image file routines
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "db_vm_v010.h"
#include "XGS_PIC_UART_DRV_V010.h"

/* LoadImage - load an image file */
int LoadImage(Interpreter *i, char *name)
{
    size_t size;
    FileHdr hdr;
    VMFILE *fp;

    /* open the image file */
    if (!(fp = (*i->fileOpen)(name, "rb"))) {
        VM_printf("file open failed\n");
        return VMFALSE;
    }

    /* read the image header */
    if ((*i->fileRead)(&hdr, sizeof(hdr), fp) != sizeof(hdr)) {
        VM_printf("header read failed\n");
        goto fail;
    }

    /* verify the image header */
    if (strncmp(hdr.tag, FileTag, strlen(FileTag)) != 0 || hdr.version != FileVersion) {
        VM_printf("header verify failed\n");
        goto fail;
    }

    /* verify the target */
    if (hdr.target != i->target) {
        VM_printf("not built for this target: expected %d, got %d\n", i->target, hdr.target);
        goto fail;
    }

    /* allocate the code and data space */
    size = (hdr.variableCount + hdr.stackSize) * sizeof(int16_t);
    if (!AllocateImage(i, (int)hdr.codeSize, (int)size)) {
        VM_printf("image space allocation failed\n");
        goto fail;
    }

    /* read the code */
    (*i->fileSeek)(fp, hdr.codeOffset, SEEK_SET);
    if ((*i->fileRead)(i->code, hdr.codeSize, fp) != hdr.codeSize) {
        VM_printf("code read failed\n");
        goto fail2;
    }

    /* read the variable initial values */
    (*i->fileSeek)(fp, hdr.variableOffset, SEEK_SET);
    size = hdr.variableCount * sizeof(int16_t);
    if ((*i->fileRead)(i->variables, size, fp) != size) {
        VM_printf("variable read failed\n");
        goto fail2;
    }

    /* close the image file */
    (*i->fileClose)(fp);

    /* get the offset to the main code */
    i->mainCodeOffset = hdr.mainCodeOffset;
    i->stack = (int16_t *)(i->variables + hdr.variableCount);
    i->stackTop = i->stack + hdr.stackSize;

    /* return successfully */
    return VMTRUE;

    /* cleanup and report a failure */
fail2:
    FreeImage(i);
fail:
    (*i->fileClose)(fp);
    return VMFALSE;
}

