'' Paddle interface - uses Analog to Digtal interface on the C3 board
'' JT Cook
CON
  'use this to clip the edge of potentiometer for weird fall off
  pot_edge0 = 30 'channel/paddle 0
  pot_edge1 = 30 'channel/paddle 1
  ' SPI pins
  SPI_SEL_CLK            = 8  ' used to clock the SPI counter that feeds 4-bit value to SPI selection decoder, clocks on rising edge
  SPI_SEL_CLR            = 25 ' used to clear the SPI counter back to 0000b (NULL SPI device), active LOW

  SPI_MOSI               = 9  ' SPI master out serial in to slave
  SPI_MISO               = 10 ' SPI master in serial out from slave, 10 for good C3, change to 7 for BROKEN C3 with YELLOW jumper.
  SPI_SCK                = 11 ' SPI clock from master to all slaves

  ' SPI channels defines 
  SPI_CHAN_NULL          = 0 ' NULL channel, disables all on/off board devices.
  SPI_CHAN_SRAM0         = 1 ' 32K SRAM Bank 0.
  SPI_CHAN_SRAM1         = 2 ' 32K SRAM Bank 1.
  SPI_CHAN_FLASH         = 3 ' 1MB FLASH Memory.
  SPI_CHAN_AD            = 4 ' MCP3202 2-Channel 12-bit A/D.
  SPI_CHAN_SD            = 5 ' Micro SD Card.

PUB read_paddle0| p_temp1      
    ' read channel 0 - return an 8 bit value
    SPI_Select_Channel( SPI_CHAN_AD )
    p_temp1 := (SPI_Write_Read( 17, %0001_10_1_0000000000000, $FFF )>>4)
    'on the low end it falls off oddly
    if(p_temp1<pot_edge0)
       p_temp1:=0
    else
       p_temp1-=pot_edge0
    SPI_Select_Channel( 0 )
    return(p_temp1)
' /////////////////////////////////////////////////////////////////////////////
PUB read_paddle1| p_temp1
    ' read channel 1 - return an 8 bit value
    SPI_Select_Channel( SPI_CHAN_AD )
    p_temp1 := (SPI_Write_Read( 17, %0001_11_1_0000000000000, $FFF )>>4)
    'on the low end it falls off oddly
    if(p_temp1<pot_edge1)
       p_temp1:=0
    else
       p_temp1-=pot_edge1
    
    SPI_Select_Channel( 0 )
    return(p_temp1)
' /////////////////////////////////////////////////////////////////////////////
PUB read_buttons | buttons
 'read buttons from each paddle - only 2 bits
 'bit 0 is button first paddle, bit 1 is button on 2nd paddle
 buttons:=0 ' not yet implemented
 return (buttons & 3)
' /////////////////////////////////////////////////////////////////////////////
PUB Start
  ' set SPI mux counter IOs up
  DIRA[SPI_SEL_CLK] := 1 ' set to output
  OUTA[SPI_SEL_CLK] := 0
  

  DIRA[SPI_SEL_CLR] := 1 ' set to output
  OUTA[SPI_SEL_CLR] := 0 ' CLR counter
  OUTA[SPI_SEL_CLR] := 1 ' allow counting

  ' set up SPI lines
  OUTA[SPI_MOSI]     := 0 ' set to LOW
  OUTA[SPI_SCK]      := 0 ' set to LOW
 
  DIRA[SPI_MOSI]     := 1 ' set to output
  DIRA[SPI_MISO]     := 0 ' set to input
  DIRA[SPI_SCK]      := 1 ' set to output


PUB SPI_Write_Read( num_bits, data_out, bit_mask) | data_in
{{
This function writes and reads SPI data a bit at a time (SPI is a circular buffer protocal), the data is in MSB to LSB format
and up to 32-bits can be transmitted and received, the final result is bit masked by bit_mask

PARMS:

num_bits : number of bits to transmit from data_out
data_out : source of data to transmit
bit_mask : final result of SPI transmission is masked with this to grab the relevant least significant bits

RETURNS: data retrieved from SPI transmission

}}
  ' clear result
  data_in := 0

  ' now read the bits in/out
  repeat num_bits
    ' drop clock
    OUTA[SPI_SCK] := 0

    ' place next bit on MOSI
    OUTA[SPI_MOSI] := ((data_out >> (num_bits-- - 1)) & $01)
 
    ' now read next bit from MISO
    data_in := (data_in << 1) + INA[SPI_MISO]
      
    ' raise clock
    OUTA[SPI_SCK] := 1

  ' set clock and MOSI to LOW on exit
  OUTA[SPI_MOSI]  := 0
  OUTA[SPI_SCK]   := 0

  ' at this point, the data has been written and read, return result
  return ( data_in & bit_mask )            
    
' end SPI_Write_Read

' /////////////////////////////////////////////////////////////////////////////

PUB SPI_Select_Channel( channel )
{{
This function sets the active SPI channel chip select on the SPI mux, this is accomplished by first resetting the
SPI counter that feeds the SPI select decoder, then upcounting the requested number of channels.

PARMS:

channel : channel 0 to 7 to enable where the channels are defined as follows
          0 - NULL channel, disables all on/off board devices.
          1 - 32K SRAM Bank 0.
          2 - 32K SRAM Bank 1.
          3 - 1MB FLASH Memory.
          4 - MCP3202 2-Channel 12-bit A/D.
          5 - Micro SD Card.
          6 - Header Interface SPI6.
          7 - Header Interface SPI7.

RETURNS: nothing.

}}

  ' requesting channel 0? If so, easy reset
  if (channel == 0)
    ' clear the 161
    OUTA[SPI_SEL_CLR] := 0 ' CLR counter
    OUTA[SPI_SEL_CLR] := 1 ' allow counting
    return

  ' else non-null channel, count up to channel...
  
  ' first reset the SPI channel counter
  ' clear the 161
  OUTA[SPI_SEL_CLR] := 0 ' CLR counter
  OUTA[SPI_SEL_CLR] := 1 ' allow counting

  ' now increment to requested channel
  ' clock the 161
   OUTA[SPI_SEL_CLK] := 0

   repeat channel
     OUTA[SPI_SEL_CLK] := 1
     OUTA[SPI_SEL_CLK] := 0        

' end SPI_Select_Channel

' /////////////////////////////////////////////////////////////////////////////
