'
' based on code from VMCOG - virtual memory server for the Propeller
'
' Copyright February 3, 2010 by William Henning
'
' This code is released under the terms of the MIT license, "Attribution Required", and this
' Copyright notice may not be removed. Basically this means that as long as you leave this
' copyright notice in any code that uses this file, and provide credit to the authors in
' the source and documentation for your product, you may use this code. Please note that
' this does NOT give you any license to the hardware the code may be written for - for example,
' if there is an XMM driver for Acme board's XMM interface, it does not give you the right to
' duplicate the Acme board, or its XMM interface.
'

'#define HYDRA_SPI_SRAM
#define C3

CON

  READ_CMD              = 3
  WRITE_CMD             = 2
  LINELEN               = 1<<7  ' FIX ME!!!

#ifdef HYDRA_SPI_SRAM

  ' pins used by first SPI SRAM
  CS_PIN                = 19
  CLK_PIN               = 17
  MOSI_PIN              = 16
  MISO_PIN              = 18
  
  ' pins used by second SPI SRAM
  CS2_PIN               = 20
  CLK2_PIN              = 17
  MOSI2_PIN             = 16
  MISO2_PIN             = 18

#endif

#ifdef C3

  CLR_PIN               = 25
  INC_PIN               = 8
  CLK_PIN               = 11
  MOSI_PIN              = 9
  MISO_PIN              = 10

#endif

  ' default cache dimensions
  DEFAULT_INDEX_WIDTH   = 5
  DEFAULT_OFFSET_WIDTH  = 7

  ' cache line tag flags
  EMPTY_BIT             = 30
  DIRTY_BIT             = 31
 
VAR

   long init_params[4]
  
PUB start(mailbox)
   init_params[0] := mailbox
   init_params[1] := $6e00
   init_params[2] := 5
   init_params[3] := 7
   long[mailbox] := $ffffffff
   cognew(@init_vm, @init_params)
   repeat while long[mailbox]

DAT
        org   $0

' initialization structure offsets
' $0: pointer to a two word mailbox
' $4: pointer to where to store the cache lines in hub ram
' $8: number of bits in the cache line index if non-zero (default is DEFAULT_INDEX_WIDTH)
' $a: number of bits in the cache line offset if non-zero (default is DEFAULT_OFFSET_WIDTH)
' note that $4 must be at least 2^($8+$a) bytes in size
' the cache line mask is returned in $0

init_vm mov     t1, par             ' get the address of the initialization structure
        rdlong  pvmcmd, t1          ' pvmcmd is a pointer to the virtual address and read/write bit
        mov     pvmaddr, pvmcmd     ' pvmaddr is a pointer into the cache line on return
        add     pvmaddr,  #4
        add     t1, #4
        rdlong  cacheptr, t1        ' cacheptr is the base address in hub ram of the cache
        add     t1, #4
        rdlong  t2, t1 wz
  if_nz mov     index_width, t2     ' override the index_width default value
        add     t1, #4
        rdlong  t2, t1 wz
  if_nz mov     offset_width, t2    ' override the offset_width default value

        mov     index_count, #1
        shl     index_count, index_width
        mov     index_mask, index_count
        sub     index_mask, #1

        mov     line_size, #1
        shl     line_size, offset_width
        mov     t1, line_size
        sub     t1, #1
        wrlong  t1, par
        shr     line_size, #2

#ifdef HYDRA_SPI_SRAM

        ' set the pin directions
        mov     dira, spidir

        ' first chip
        call    #chip1
        mov     data, ramseq
        mov     bits,#16
        andn    outa, TCS_CLK
        call    #send
        or      outa, TCS

        ' second chip
        call    #chip2
        mov     data, ramseq
        mov     bits, #16
        andn    outa, TCS_CLK
        call    #send
        or      outa, TCS

#endif

#ifdef C3

        ' set the pin directions
        mov     dira, spidir
        or      outa, TCLR
        andn    outa, TCLK

        ' first chip
        call    #chip1
        mov     data, ramseq
        mov     bits,#16
        andn    outa, TCLK
        call    #send
        andn    outa, TCLR
        or      outa, TCLR

        ' second chip
        call    #chip2
        mov     data, ramseq
        mov     bits, #16
        andn    outa, TCLK
        call    #send
        andn    outa, TCLR
        or      outa, TCLR

#endif

        jmp     #vmflush

fillme  long    0[128-fillme]           ' first 128 cog locations are used for a direct mapped page table

        fit   128

        ' initialize the cache lines
vmflush movd    :flush, #0
        mov     t1, index_count
:flush  mov     0-0, empty_mask
        add     :flush, dstinc
        djnz    t1, #:flush

        ' start the command loop
waitcmd wrlong  zero, pvmcmd
        mov     dira, #0                ' release the pins for other SPI clients

:wait   rdlong  vmpage, pvmcmd wz
        shr     vmpage, offset_width wc ' carry is now one for read and zero for write
  if_z  jmp     #:wait
        mov     dira, spidir            ' set the pins back so we can use them

        mov     set_dirty_bit, #0       ' make mask to set dirty bit on writes
        muxnc   set_dirty_bit, dirty_mask
        mov     line, vmpage            ' get the cache line index
        and     line, index_mask
        mov     hubaddr, line
        shl     hubaddr, offset_width
        add     hubaddr, cacheptr       ' get the address of the cache line
        wrlong  hubaddr, pvmaddr        ' return the address of the cache line
        movs    :ld, line
        movd    :st, line
:ld     mov     vmcurrent, 0-0          ' get the cache line tag
        and     vmcurrent, tag_mask
        cmp     vmcurrent, vmpage wz    ' z set means there was a cache hit
  if_nz call    #miss                   ' handle a cache miss
:st     or      0-0, set_dirty_bit      ' set the dirty bit on writes
        jmp     #waitcmd                ' wait for a new command

' line is the cache line index
' vmcurrent is current page
' vmpage is new page
' hubaddr is the address of the cache line
miss    movd    :test, line
        movd    :st, line
:test   test    0-0, dirty_mask wz
  if_z  jmp     #:rd                    ' current page is clean, just read new page
        mov     vmaddr, vmcurrent
        shl     vmaddr, offset_width
        call    #BWRITE                 ' write current page
:rd     mov     vmaddr, vmpage
        shl     vmaddr, offset_width
        call    #BREAD                  ' read new page
:st     mov     0-0, vmpage
miss_ret ret

' pointers to mailbox entries
pvmcmd          long    0       ' on call this is the virtual address and read/write bit
pvmaddr         long    0       ' on return this is the address of the cache line containing the virtual address

cacheptr        long    0       ' address in hub ram where cache lines are stored
vmpage          long    0       ' page containing the virtual address
vmcurrent       long    0       ' current page in selected cache line (same as vmpage on a cache hit)
line            long    0       ' current cache line index
set_dirty_bit   long    0       ' DIRTY_BIT set on writes, clear on reads

zero            long    0       ' zero constant
dstinc          long    1<<9    ' increment for the destination field of an instruction
t1              long    0       ' temporary variable
t2              long    0       ' temporary variable

tag_mask        long    !(1<<DIRTY_BIT) ' includes EMPTY_BIT
index_width     long    DEFAULT_INDEX_WIDTH
index_mask      long    0
index_count     long    0
offset_width    long    DEFAULT_OFFSET_WIDTH
line_size       long    0                       ' line size in longs
empty_mask      long    (1<<EMPTY_BIT)
dirty_mask      long    (1<<DIRTY_BIT)

'----------------------------------------------------------------------------------------------------
'
' BSTART
'
' Assert /CS for the device specified by "vmaddr", initially 2 pins are used to select RAM0 or RAM1
' (the MCP23K256's are 32KB devices), later can choose between different XMM's on same prop :)
'
'----------------------------------------------------------------------------------------------------

BSTART
        test    vmaddr, bit16 wz
  if_z  call    #chip1      ' drop /CS, drop CLK
  if_nz call    #chip2      ' drop /CS, drop CLK

        mov     data, vmaddr
        and     data, offs_mask   ' limit offset to $7FFF (32k-1)
        shl     data, #8          ' move it into position for transmission
        or      data, fn
        mov     bits, #24
#ifdef HYDRA_SPI_SRAM
        andn    outa, TCS_CLK     ' drop /CS, drop CLK
#endif
#ifdef C3
        andn    outa, TCLK        ' drop CLK
#endif
        call    #send

        mov     ptr, hubaddr      ' hubaddr = hub page address
        mov     count, line_size
BSTART_RET      ret


'----------------------------------------------------------------------------------------------------
'
' BREAD
'
' vmaddr is the virtual memory address to read
' hubaddr is the hub memory address to write
'
' trashes fn, count, bits, data, ptr, count, c and z flags
'
'----------------------------------------------------------------------------------------------------

BREAD
        mov     fn, read
        call    #BSTART

pr      mov     bits, #32

gloop   or      outa, TCLK
        test    TMISO, ina wc
        rcl     data, #1
        andn    outa, TCLK
        djnz    bits, #gloop

        wrlong  data, ptr
        add     ptr, #4
        djnz    count, #pr

#ifdef HYDRA_SPI_SRAM
        or      outa, TCS
#endif
#ifdef C3
        andn    outa, TCLR
        or      outa, TCLR
#endif
BREAD_RET     ret

'----------------------------------------------------------------------------------------------------
'
' BWRITE
'
' vmaddr is the virtual memory address to write
' hubaddr is the hub memory address to read
'
' trashes fn, count, bits, data, ptr, count, z flag
'
'----------------------------------------------------------------------------------------------------

BWRITE
        mov     fn, write
        call    #BSTART

pw      rdlong  data, ptr
        mov     bits, #32
        call    #send
        add     ptr, #4
        djnz    count, #pw

#ifdef HYDRA_SPI_SRAM
        or      outa, TCS
#endif
#ifdef C3
        andn    outa, TCLR
        or      outa, TCLR
#endif
BWRITE_RET      ret

fn          long    0

'----------------------------------------------------------------------------------------------------
' SPI routines
'----------------------------------------------------------------------------------------------------

send    andn    outa, TCLK
        rol     data, #1 wc
        muxc    outa, TMOSI
        or      outa, TCLK
        djnz    bits, #send
        nop
        andn    outa, TCLK_MOSI
send_ret ret

#ifdef HYDRA_SPI_SRAM

chip1   mov     tcs, cs
        mov     tclk, clk
        mov     tmosi, mosi
        mov     tmiso, miso
        mov     tcs_clk, cs_clk
        mov     tclk_mosi, clk_mosi
chip1_ret ret

chip2   mov     tcs, cs2
        mov     tclk, clk2
        mov     tmosi, mosi2
        mov     tmiso, miso2
        mov     tcs_clk, cs2_clk2
        mov     tclk_mosi, clk2_mosi2
chip2_ret ret

spidir      long    (1<<CS_PIN)|(1<<CLK_PIN)|(1<<MOSI_PIN)|(1<<CS2_PIN)|(1<<CLK2_PIN)|(1<<MOSI2_PIN)

cs          long    1<<CS_PIN
clk         long    1<<CLK_PIN
mosi        long    1<<MOSI_PIN
miso        long    1<<MISO_PIN
cs_clk      long    (1<<CS_PIN)|(1<<CLK_PIN)
clk_mosi    long    (1<<CLK_PIN)|(1<<MOSI_PIN)

cs2         long    1<<CS2_PIN
clk2        long    1<<CLK2_PIN
mosi2       long    1<<MOSI2_PIN
miso2       long    1<<MISO2_PIN
cs2_clk2    long    (1<<CS2_PIN)|(1<<CLK2_PIN)
clk2_mosi2  long    (1<<CLK2_PIN)|(1<<MOSI2_PIN)

tcs         long    0
tclk        long    0
tmosi       long    0
tmiso       long    0
tcs_clk     long    0
tclk_mosi   long    0

#endif

#ifdef C3

' spi select 1
chip1   andn    outa, TCLR
        or      outa, TCLR
        or      outa, TINC
        andn    outa, TINC
chip1_ret ret

' spi select 2
chip2   andn    outa, TCLR
        or      outa, TCLR
        or      outa, TINC
        andn    outa, TINC
        or      outa, TINC
        andn    outa, TINC
chip2_ret ret

spidir      long    (1<<CLR_PIN)|(1<<INC_PIN)|(1<<CLK_PIN)|(1<<MOSI_PIN)

tclr        long    1<<CLR_PIN
tinc        long    1<<INC_PIN
tclk        long    1<<CLK_PIN
tmosi       long    1<<MOSI_PIN
tmiso       long    1<<MISO_PIN
tclk_mosi   long    (1<<CLK_PIN)|(1<<MOSI_PIN)

#endif

' input parameters to BREAD and BWRITE
vmaddr      long    0       ' virtual address
hubaddr     long    0       ' hub memory address to read from or write to

' temporaries used by BREAD and BWRITE
ptr         long    0
count       long    0

' temporaries used by send
bits        long    0
data        long    0

read        long    $03000000       ' read command
write       long    $02000000       ' write command
ramseq      long    $01400000       ' %00000001_01000000 << 16 ' set sequetial mode
readstat    long    $05000000       ' read status

offs_mask   long    $7FFF
bit16       long    $8000

          FIT 496               ' out of 496
