CON

  _clkmode              = xtal1 + pll16x
  _xinfreq              = 5_000_000
  
  baudrate              = 115200

  PAGE_SIZE             = vm#LINELEN
  PAGE_SIZE_IN_WORDS    = PAGE_SIZE / 2
  PAGE_SIZE_IN_LONGS    = PAGE_SIZE / 4
  PAGE_MASK             = PAGE_SIZE - 1
  PAGE_COUNT            = 65536 / PAGE_SIZE

VAR

  word buf1[PAGE_SIZE_IN_WORDS]
  word buf2[PAGE_SIZE_IN_WORDS]

  long mailbox[2]

OBJ

  ser : "FullDuplexSerialPlus"
  vm  : "spisram_cache"

PUB Start | ch

  ser.start(31, 30, 0, baudrate)
  vm.start(@mailbox)

  repeat
    show(@menu)
    ch := ser.rx
    ser.tx($d)
    ser.tx($a)
    case ch
      "c":   ClearPageCmd
      "f":   FillPageCmd(0)
      "s":   ShowPageCmd
      "1":   Test1Cmd
      "2":   Test2Cmd
      other: ser.str(@unknown)

PRI ClearPageCmd | page, i
  page := get_value(string("Page? "))
  repeat i from 0 to PAGE_SIZE_IN_WORDS-1
    buf1[i] := 0
  WritePage(page*PAGE_SIZE, @buf1)

PRI FillPageCmd(value) | page, i
  page := get_value(string("Page? "))
  repeat i from 0 to PAGE_SIZE_IN_WORDS-1
    buf1[i] := value++
  WritePage(page*PAGE_SIZE, @buf1)

PRI ShowPageCmd | page, i, j
  page := get_value(string("Page? "))
  ReadPage(page*PAGE_SIZE, @buf2)
  repeat j from 0 to PAGE_SIZE_IN_WORDS/4-1
    ser.hex(page*PAGE_SIZE+j*8, 4)
    ser.str(STRING(": "))
    repeat i from 0 to 3
      ser.hex(buf2[j*4+i], 4)
      ser.tx(" ")
    ser.tx($d)
    ser.tx($a)

PRI Test1Cmd | i, j, value

  value := 0
  repeat j from 0 to PAGE_COUNT-1
    repeat i from 0 to PAGE_SIZE_IN_WORDS-1
      buf1[i] := value++
    WritePage(j*PAGE_SIZE, @buf1)

  value := 0
  repeat j from 0 to PAGE_COUNT-1
    ReadPage(j*PAGE_SIZE, @buf2)
    repeat i from 0 to PAGE_SIZE_IN_WORDS-1
      if buf2[i] <> value++
        ser.hex(buf2[i], 4)
        ser.tx(" ")
        ser.hex(value-1, 4)
        ser.tx($d)
        ser.tx($a)
  ser.str(STRING($d, $a, "Test1 done", $d, $a))

PRI Test2Cmd | i, j, value

  value := 0
  repeat j from 0 to PAGE_COUNT-1
    repeat i from 0 to PAGE_SIZE_IN_WORDS-1
      buf1[i] := !value++
    WritePage(j*PAGE_SIZE, @buf1)

  value := 0
  repeat j from 0 to PAGE_COUNT-1
    ReadPage(j*PAGE_SIZE, @buf2)
    repeat i from 0 to PAGE_SIZE_IN_WORDS-1
      if buf2[i] <> !value++ & $ffff
        ser.hex(buf2[i], 4)
        ser.tx(" ")
        ser.hex(value-1, 4)
        ser.tx($d)
        ser.tx($a)
  ser.str(STRING($d, $a, "Test2 done", $d, $a))

PUB get_value(prompt) : t
  ser.str(prompt)
  t := ser.GetHex
  ser.tx($d)
  ser.tx($a)

PRI show(p)
  repeat until (byte[p]==0)
    ser.str(p)
    p += strsize(p) + 1

PRI ReadPage(addrs, buf)
  mailbox[0] := (addrs&!3)|vm#READ_CMD
  repeat while mailbox[0] <> 0
  longmove(buf, mailbox[1], PAGE_SIZE_IN_LONGS)

PRI WritePage(addrs, buf)
  mailbox[0] := (addrs&!3)|vm#WRITE_CMD
  repeat while mailbox[0] <> 0
  longmove(mailbox[1], buf, PAGE_SIZE_IN_LONGS)

DAT

menu    byte $d,$a,"C3 SPI SRAM Main Menu",$d,$a,$d,$a,0
        byte "c) Clear Page",$d,$a,0
        byte "f) Fill Page",$d,$a,0
        byte "s) Show Page",$d,$a,0
        byte "1) Test 1 - normal",$d,$a,0
        byte "2) Test 2 - inverse",$d,$a,0
        byte $d,$a,"CMD>",0,0

unknown byte "ERROR: Unknown Command",0

