{{
Modified Tiny Basic for use with Propeller Demo Board and Hydra.
Original Tiny Basic written by Tomas Rokicki & Radical Eye Software.

Copyright (c) 2010 Michael Green.  See end of file for terms of use.
}}

'' 2009-07-09 - Corrected WRITE and PRINT for case of -2,147,483,648
'' 2010-06-04 - Changed fsrw to Rokicki's version 2.6 for FAT16/FAT32
'' 2010-07-18 - Changed to support Andre LaMothe's Prop C3 board
''              sdspiFemto changed to use counter to produce SPI CS.
''              fsrwFemto changed to support flash file system.
''              VGA enable set properly for either TV or VGA use.
''              statements and pseudo-variables added to support
''              SRAM, flash, and an MCP3202 ADC.
'' 2010-09-15 - Added some temporary debugging messages
'' 2010-09-20 - Removed debugging messages
''              Changed call to Winbond object via fsrwFemto
'' 2010-10-02 - Corrected fsrwFemto to use .pputc for .out
'' 2010-10-13 - Added display of flash size.  Corrected 1MB JEDEC code
'' 2010-10-14 - Added option for hex formatted output with PRINT and WRITE
'' 2010-11-05 - Added support for flash file system in Winbond
'' 2010-11-06 - Changed I/O to Sphinx style
'' 2010-11-19 - Added FILES file matching
'' 2010-11-21 - Bug fixes and added flash file loader
'' 2010-12-02 - Bug fixes
'' 2010-12-05 - Added EXIT statement to reboot (Sphinx may not be active)

obj
   drv  : "isxfs"                      ' FAT File System Read/Write
   drvx : "sxfs"                       '  (starts up low level SPI)
   spi  : "isxspi"                     ' Driver for other SPI devices
   dsp  : "isxtv"                      ' Sphinx TV Text Driver
   dspx : "sxtv"
   key  : "isxkb"                      ' Sphinx Keyboard Driver
   keyx : "sxkb"
   i2c  : "basici2c"                   ' I2C EEPROM Driver

con
   version   = 1                       ' Major version
   release   = 004                     ' Minor release
   testLevel = 0                       ' Test change level

   tvPins    = 12                      ' TV I/O pins +0 to +2
   kbPins    = 26                      ' PS/2 I/O pins +0 and +1
   sdPinsCnt = 8  << 20                ' SPI increment select counter
   sdPinsDO  = 9  << 0                 ' SPI MISO data from device
   sdPinsDI  = 10 << 5                 ' SPI MOSI data to device
   sdPinsClk = 11 << 10                ' SPI clock
   sdPinsClr = 25 << 15                ' SPI /clear select counter
   sdPinsSel = 5  << 25                ' SD card select code
   sdPins    = sdPinsDO+sdPinsDI+sdPinsClk+sdPinsClr+sdPinsCnt+sdPinsSel
   
   progsize  = 4096                    ' Space reserved for program
   _clkmode  = xtal1 + pll16x
   _xinfreq  = 5_000_000
   _stack    = 100                     ' Roughly 4 nested expressions
   userPtr   = drv#METADATABUFFER - 4
   _free     = (($8000 - userPtr) + progsize + $14) / 4

   bspKey    = $C8                     ' PS/2 keyboard backspace key
   breakKey  = $CB                     ' PS/2 keyboard escape key
   backspace = 8
   fReturn   = 13
   fLinefeed = 10
   fEof      = -1
      
   maxstack  = 20                      ' Maximum stack depth
   linelen   = 256                     ' Maximum input line length
   quote     = 34                      ' Double quote
   caseBit   = !32                     ' Uppercase/Lowercase bit
   vBase     = $0008                   ' Variables area base pointer

   SXTVRENDEZVOUS  = $8000 - 4               ' TV display driver area
   SXKBRENDEZVOUS  = SXTVRENDEZVOUS - 4      ' PS/2 keyboard driver area
   SDSPIRENDEZVOUS = SXKBRENDEZVOUS - 3 * 4  ' Low level SPI driver area
   SXFS2RENDEZVOUS = SDSPIRENDEZVOUS - 4 * 4 ' File system driver #2 area
   SXFSRENDEZVOUS  = SXFS2RENDEZVOUS - 4 * 4 ' File system driver #1 area
   METADATABUFFER  = SXFSRENDEZVOUS - 512    ' Metadata area for file system

var
   long sp, tp, eop, nextlineloc, rv, curlineno, pauseTime
   long vars[26], stack[maxstack], control[2]
   long forStep[26], forLimit[26], forLoop[26]
   long ioControl[2], fcb[4], fileOpened
   long fileStuff[drv#SIZEOFFILESTUFF/4]
   word outputs
   byte tline[linelen], tailLine[linelen], inVars[26]
   
   word bytesPerSector
   byte sectorsPerCluster
   word reservedSectors
   byte numberOfFats
   word sectorsPerFat
   word numberOfRootDirectoryEntries
   long fat1[0]
   long fatSector0
   long dirSector0
   long dataregion[0] 
   long dataSector0

dat
   tok0  byte "IF", 0
   tok1  byte "THEN", 0
   tok2  byte "INPUT", 0    ' INPUT {"<prompt>";} <var> {,<var>}
   tok3  byte "PRINT", 0    ' PRINT {USING "<format>";} ...
   tok4  byte "GOTO", 0
   tok5  byte "GOSUB", 0
   tok6  byte "RETURN", 0
   tok7  byte "REM", 0
   tok8  byte "NEW", 0
   tok9  byte "LIST", 0
   tok10 byte "RUN", 0
   tok11 byte "RND", 0
   tok12 byte "OPEN", 0     ' OPEN " <file> ",<mode>
   tok13 byte "READ", 0     ' READ <var> {,<var>}
   tok14 byte "WRITE", 0    ' WRITE {USING "<format>";} ...
   tok15 byte "CLOSE", 0    ' CLOSE
   tok16 byte "DELETE", 0   ' DELETE " <file> "
   tok17 byte "RENAME", 0   ' RENAME " <file> "," <file> "
   tok18 byte "FILES", 0    ' FILES {" <file> "}
   tok19 byte "SAVE", 0     ' SAVE or SAVE [<expr>] or SAVE "<file>"
   tok20 byte "LOAD", 0     ' LOAD or LOAD [<expr>] or LOAD "<file>"
   tok21 byte "NOT" ,0      ' NOT <logical>
   tok22 byte "AND" ,0      ' <logical> AND <logical>
   tok23 byte "OR", 0       ' <logical> OR <logical>
   tok24 byte "SHL", 0      ' <expr> SHL <expr>
   tok25 byte "SHR", 0      ' <expr> SHR <expr>
   tok26 byte "FOR", 0      ' FOR <var> = <expr> TO <expr>
   tok27 byte "TO", 0
   tok28 byte "STEP", 0     '  optional STEP <expr>
   tok29 byte "NEXT", 0     ' NEXT <var>
   tok30 byte "INA", 0      ' INA [ <expr> ]
   tok31 byte "OUTA", 0     ' OUTA [ <expr> ] = <expr>
   tok32 byte "PAUSE", 0    ' PAUSE <time ms> {,<time us>}
   tok33 byte "USING", 0    ' PRINT USING "<format>"; ...
   tok34 byte "ROL", 0      ' <expr> ROL <expr>
   tok35 byte "ROR", 0      ' <expr> ROR <expr>
   tok36 byte "SAR", 0      ' <expr> SAR <expr>
   tok37 byte "REV", 0      ' <expr> REV <expr>
   tok38 byte "BYTE", 0     ' BYTE [ <expr> ]
   tok39 byte "WORD", 0     ' WORD [ <expr> ]
   tok40 byte "LONG", 0     ' LONG [ <expr> ]
   tok41 byte "CNT", 0
   tok42 byte "PHSA", 0
   tok43 byte "PHSB", 0
   tok44 byte "FRQA", 0
   tok45 byte "FRQB", 0
   tok46 byte "CTRA", 0
   tok47 byte "CTRB", 0
   tok48 byte "DISPLAY", 0  ' DISPLAY <expr> {,<expr>}
   tok49 byte "KEYCODE", 0  ' KEYCODE
   tok50 byte "LET", 0
   tok51 byte "STOP", 0
   tok52 byte "END", 0
   tok53 byte "EEPROM", 0   ' EEPROM[ <expr> ]
   tok54 byte "FILE", 0     ' FILE
   tok55 byte "MEM", 0      ' MEM
   tok56 byte "SPIN", 0     ' SPIN [<expr>] or SPIN "<file>"
   tok57 byte "COPY", 0     ' COPY [<expr>],"<file>" or COPY "<file>",[<expr>] or
                            ' COPY [<expr>],<expr> where <expr> are different
   tok58 byte "DUMP", 0     ' DUMP <expr>,<expr> or DUMP [<expr>],<expr>
   tok59 byte "SRAM", 0     ' SRAM [<expr> {,<expr>}]
   tok60 byte "ERASE", 0    ' ERASE <expr>
   tok61 byte "FLASH", 0    ' FLASH [<expr> {,<expr>}]
   tok62 byte "ADC", 0      ' ADC [<expr>]
   tok63 byte "FCB", 0      ' FCB [<expr>] or FCB[<expr>] = <expr>
   tok64 byte "EXIT", 0     ' EXIT

   toks  word @tok0, @tok1, @tok2, @tok3, @tok4, @tok5, @tok6, @tok7
         word @tok8, @tok9, @tok10, @tok11, @tok12, @tok13, @tok14, @tok15
         word @tok16, @tok17, @tok18, @tok19, @tok20, @tok21, @tok22, @tok23
         word @tok24, @tok25, @tok26, @tok27, @tok28, @tok29, @tok30, @tok31
         word @tok32, @tok33, @tok34, @tok35, @tok36, @tok37, @tok38, @tok39
         word @tok40, @tok41, @tok42, @tok43, @tok44, @tok45, @tok46, @tok47
         word @tok48, @tok49, @tok50, @tok51, @tok52, @tok53, @tok54, @tok55
         word @tok56, @tok57, @tok58, @tok59, @tok60, @tok61, @tok62, @tok63
         word @tok64
   tokx  word

   syn   byte "Syntax Error", 0
   ln    byte "Invalid Line Number", 0

PUB main | err, s
   if dspx.start( tvPins, SXTVRENDEZVOUS )       ' Start TV driver
      keyx.start( kbPins )                       ' Start keyboard
      drvx.start( sdPins )                       ' Start SPI drivers
   dsp.str(string("C3 FemtoBasic"))
   if version > 0 or release > 0 or testLevel > 0
      dsp.str(string(" Version "))
      dsp.dec(version)
      dsp.out(".")
      if release < 100
         dsp.out("0")
      if release < 10
         dsp.out("0")
      dsp.dec(release)
      if testLevel > 0
         dsp.out("a"+testLevel-1)
   dsp.out(fReturn)
   dsp.str(string("Flash: "))
   dsp.dec((spi.start * spi#secSize) >> 10)
   dsp.str(string("K free out of "))
   dsp.dec(spi.flashSize >> 20)
   dsp.str(string("MB",fReturn))
   longfill(@fcb,-1,4)                           ' Initialize fcb
   pauseTime := 0
   outputs := 0
   fileOpened := 0
   long[userPtr] := userPtr - progsize ' Allocate memory
   clearall                                      ' Clear program and vars
   s := 0
   curlineno := -1
   ifnot \drv.Open(@fileStuff,string("autoexec.bas"),"R")
      if (err := \processLoad) <> fEof
         showError(err)
         newprog
      \drv.Close(@fileStuff)
      if err == fEof
         s := string("run")
   ifnot s                                       ' No successful RUN?
      if spi.initFile(@fcb,string("autoexec.bas"))
         if spi.openFile(@fcb)                   ' Try flash memory
            if (err := \processLoad) <> fEof
               showError(err)
               newprog
            longfill(@fcb,-1,4)                  ' Close FCB
            if err == fEof
               s := string("run")
   repeat
      err := \doline(s)
      s := 0
      if err
         showError(err)

PRI showError(err)
   if curlineno => 0
      dsp.str(string("IN LINE "))
      dsp.dec(curlineno)
      dsp.out(" ")
   putlinet(err)
   nextlineloc := eop - 2

pri isFlashFile                      ' Does FCB point to flash file?
   return (fcb[0] & fcb[1] & fcb[2]) <> $FFFFFFFF

pri massIn
   if isFlashFile
      ifnot spi.readFile(@fcb,@result,1)
         return -1                   ' End of file
   else
      if drv.Read(@fileStuff,@result,1) == -1
         return -1

pri massOut(c)                       ' Output to mass storage device
   if isFlashFile
      spi.writeFile(@fcb,@c,1)
   else
      \drv.Write(@fileStuff,@c,1)

pri massDec(v) | i, x                ' Output to mass storage device
   x := v == NEGX                    ' Check for max negative
   if v < 0
      v := ||(v+x)                   ' Adjust as needed
      massOut("-")                   '  and output sign
   i := 1_000_000_000                ' Initialize divisor
   repeat 10                         ' Loop for 10 digits
      if v => i
         massOut(v / i + "0" + x*(i == 1)) 'display digit
         v //= i
         result~~                    ' Flag non-zero found
      elseif result or i == 1
         massOut("0")                ' Display at least zero
      i /= 10

PRI getline | i, c
   i := 0
   repeat
      c := key.getkey
      if c == bspKey
         if i > 0
            dsp.str(string(backspace," ",backspace))
            i--
      elseif c == fReturn
         dsp.out(c)
         tline[i] := 0
         tp := @tline
         return
      elseif i < linelen-1
         dsp.out(c)
         tline[i++] := c

pri putlinet(s) | c, ntoks
   ntoks := (@tokx - @toks) / 2
   repeat while c := byte[s++]
      if c => 128
         if (c -= 128) < ntoks
            dsp.str(@@toks[c])
            if c <> 7   ' REM
               dsp.out(" ")
         else
            dsp.out("{")
            dsp.dec(c)
            dsp.out("}")
      else
         dsp.out(c)
   dsp.out(fReturn)

pri spaces | c
   repeat
      c := byte[tp]
      if c == 0 or c > " "
         return c
      tp++

pri skipspaces
   if byte[tp]
      tp++
   return spaces

pri parseliteral | r, c
   r := 0
   repeat
      c := byte[tp]
      if c < "0" or c > "9"
         return r
      r := r * 10 + c - "0"
      tp++

pri movprog(at, delta)
   if eop + delta + 2 - long[userPtr] > progsize
      abort string("NO MEMORY")
   bytemove(at+delta, at, eop-at)
   eop += delta

pri fixvar(c)
   if c => "a"
      c -= 32
   return c - "A"

pri isvar(c)
   c := fixvar(c)
   return c => 0 and c < 26

pri tokenize | tok, c, at, put, state, i, j, ntoks
   ntoks := (@tokx - @toks) / 2
   at := tp
   put := tp
   state := 0
   repeat while c := byte[at]
      if c == quote
         if state == "Q"
            state := 0
         elseif state == 0
            state := "Q"
      if state == 0
         repeat i from 0 to ntoks-1
            tok := @@toks[i]
            j := 0
            repeat while byte[tok] and ((byte[tok] ^ byte[j+at]) & caseBit) == 0
               j++
               tok++
            if byte[tok] == 0 and not isvar(byte[j+at])
               byte[put++] := 128 + i
               at += j
               if i == 7
                  state := "R"
               else
                  repeat while byte[at] == " "
                     at++
                  state := "F"
               quit
         if state == "F"
            state := 0
         else
            byte[put++] := byte[at++]
      else
         byte[put++] := byte[at++]
   byte[put] := 0

pri wordat(loc)
   return (byte[loc]<<8)+byte[loc+1]

pri findline(lineno) | at
   at := long[userPtr]
   repeat while wordat(at) < lineno
      at += 3 + strsize(at+2)
   return at

pri insertline | lineno, fc, loc, locat, newlen, oldlen
   lineno := parseliteral
   if lineno < 0 or lineno => 65535
      abort @ln
   tokenize
   fc := spaces
   loc := findline(lineno)
   locat := wordat(loc)
   newlen := 3 + strsize(tp)
   if locat == lineno
      oldlen := 3 + strsize(loc+2)
      if fc == 0
         movprog(loc+oldlen, -oldlen)
      else
         movprog(loc+oldlen, newlen-oldlen)
   elseif fc
      movprog(loc, newlen)
   if fc
      byte[loc] := lineno >> 8
      byte[loc+1] := lineno
      bytemove(loc+2, tp, newlen-2)

pri clearvars
   bytefill(@vars, 0, 26)
   pauseTime := 0
   nextlineloc := long[userPtr]
   sp := 0

pri newprog
   byte[long[userPtr]][0] := 255
   byte[long[userPtr]][1] := 255
   byte[long[userPtr]][2] := 0
   eop := long[userPtr] + 2
   nextlineloc := eop - 2
   sp := 0

pri clearall
   newprog
   clearvars

pri pushstack
   if sp => constant(maxstack-1)
      abort string("RECURSION ERROR")
   stack[sp++] := nextlineloc

pri getAddress(delim) | t
   if spaces <> "["
      abort @syn
   skipspaces
   result := expr
   if delim == "." and (result < 0 or result > 31)
      abort string("Invalid pin number")
   if delim == "." or delim == ","
      if spaces == delim
         if delim == "."             ' Handle the form <expr>..<expr>
            if byte[++tp] <> "."
               abort @syn
            result <<= 8
            skipspaces
            t := expr
            if t < 0 or t > 31
               abort string("Invalid pin number")
            result |= t | $10000
         else                        ' Handle the form <expr>,<expr>
            if result & 1 or result < 0 or result > 31
               abort string("Invalid pin number")
            skipspaces
            result := (result << 18) | (expr & $7FFFF)
      elseif delim == ","
         result := (result & $7FFFF) | (i2c#BootPin << 18)
   if spaces <> "]"
      abort @syn
   tp++

pri factor | tok, t, i
   tok := spaces
   tp++
   case tok
      "(":
         t := expr
         if spaces <> ")"
            abort @syn
         tp++
         return t
      "a".."z","A".."Z":
         return vars[fixvar(tok)]
      158: ' INA [ <expr>{..<expr>} ]
         t := getAddress(".")
         if t > $FFFF
           tok := t & $FF
           t := (t >> 8) & $FF
           repeat i from t to tok
              outputs &= ! |< i
           dira[t..tok]~
           return ina[t..tok]
         else
           outputs &= ! |< t
           dira[t]~
           return ina[t]
      166: ' BYTE [ <expr> ]
         return byte[getAddress(" ")]
      167: ' WORD [ <expr> ]
         return word[getAddress(" ")]
      168: ' LONG [ <expr> ]
         return long[getAddress(" ")]
      181: ' EEPROM [ <expr> ]
         t := getAddress(",")
         t := i2c.readByte(i2c#BootPin,i2c#EEPROM,t)
         if t == -1
            abort string("EEPROM read")
         return t & $FF
      182: ' FILE
         return massIn
      183: ' MEM
         return progsize - (eop - long[userPtr] )
      187: ' SRAM [ <expr> {,<expr>} ]
         if spaces <> "["
            abort @syn
         skipspaces
         t := expr
         if spaces == ","
           skipspaces
           i := expr
         else
           i := 1
         if spaces <> "]" or i < 1 or i > 4
            abort @syn
         tp++
         return spi.readSRAM(t,0,-i)
      189: ' FLASH [ <expr> {,<expr>} ]
         if spaces <> "["
            abort @syn
         skipspaces
         t := expr
         if spaces == ","
            skipspaces
            i := expr
         else
            i := 1
         if spaces <> "]" or i < 1 or i > 4
            abort @syn
         tp++
         return spi.readData(t,0,-i)
      190: ' ADC [ <expr> , <expr>]
         if spaces <> "["
            abort @syn
         skipspaces
         t := expr <> 0
         if spaces <> ","
            abort @syn
         skipspaces
         i := expr <> 0
         if spaces <> "]"
            abort @syn
         tp++
         return spi.getADC(t,i)
      191: ' FCB [ <expr> ]
         t := getAddress(" ")
         if t < 0 or t > 3
            abort @syn
         return fcb[t]
      169: ' CNT
         return CNT
      170: ' PHSA
         return PHSA
      171: ' PHSB
         return PHSB
      172: ' FRQA
         return FRQA
      173: ' FRQB
         return FRQB
      174: ' CTRA
         return CTRA
      175: ' CTRB
         return CTRB
      177: ' KEYCODE
         return key.key
      139: ' RND <factor>
         return (rv? >> 1) ** (factor << 1)
      "-":
         return - factor
      "!":
         return ! factor
      "$", "%", quote, "0".."9":
         --tp
         return getAnyNumber
      other:
         abort(@syn)

pri shifts | tok, t
   t := factor
   tok := spaces
   if tok == 152 ' SHL
      tp++
      return t << factor
   elseif tok == 153 ' SHR
      tp++
      return t >> factor
   elseif tok == 162 ' ROL
      tp++
      return t <- factor
   elseif tok == 163 ' ROR
      tp++
      return t -> factor
   elseif tok == 164 ' SAR
      tp++
      return t ~> factor
   elseif tok == 165 ' REV
      tp++
      return t >< factor
   else
      return t

pri bitFactor | tok, t
   t := shifts
   repeat
      tok := spaces
      if tok == "&"
         tp++
         t &= shifts
      else
         return t

pri bitTerm | tok, t
   t := bitFactor
   repeat
      tok := spaces
      if tok == "|"
         tp++
         t |= bitFactor
      elseif tok == "^"
         tp++
         t ^= bitFactor
      else
         return t

pri term | tok, t
   t := bitTerm
   repeat
      tok := spaces
     if tok == "*"
        tp++
        t *= bitTerm
     elseif tok == "/"
        if byte[++tp] == "/"
           tp++
           t //= bitTerm
        else
           t / =bitTerm
     else
        return t

pri arithExpr | tok, t
   t := term
   repeat
      tok := spaces
      if tok == "+"
         tp++
         t += term
      elseif tok == "-"
         tp++
         t -= term
      else
         return t

pri compare | op, a, b, c
   a := arithExpr
   op := 0
   spaces
   repeat
      c := byte[tp]
      case c
         "<": op |= 1
              tp++
         ">": op |= 2
              tp++
         "=": op |= 4
              tp++
         other: quit
   case op
      0: return a
      1: return a < arithExpr
      2: return a > arithExpr 
      3: return a <> arithExpr
      4: return a == arithExpr
      5: return a =< arithExpr
      6: return a => arithExpr
      7: abort string("Invalid comparison")

pri logicNot | tok
   tok := spaces
   if tok == 149 ' NOT
      tp++
      return not compare
   return compare

pri logicAnd | t, tok
   t := logicNot
   repeat
      tok := spaces
      if tok == 150 ' AND
         tp++
         t := t and logicNot
      else
         return t

pri expr | tok, t
   t := logicAnd
   repeat
      tok := spaces
      if tok == 151 ' OR
         tp++
         t := t or logicAnd
      else
         return t

pri specialExpr
   if spaces <> "="
      abort @syn
   skipspaces
   return expr

pri scanFilename(f) | c, chars
   chars := 0
   tp++ ' skip past initial quote
   repeat while (c := byte[tp++]) <> quote
      if chars++ < 31
         byte[f++] := c
   byte[f] := 0

pri texec | ht, nt, restart, thisLine, uS, a,b,c,d,e, f0,f1,f2,f3,f4,f5,f6,f7
   uS := clkfreq / 1_000_000
   thisLine := tp - 2
   restart := 1
   repeat while restart
      restart := 0
      ht := spaces
      if ht == 0
         return
      nt := skipspaces
      if isvar(ht) and nt == "="
         tp++
         vars[fixvar(ht)] := expr
      elseif ht => 128
         case ht
            128: ' THEN
               a := expr
               if spaces <> 129
                  abort string("MISSING THEN")
               skipspaces
               if not a
                  return
               restart := 1
            130: ' INPUT {"<prompt>";} <var> {, <var>}
               if nt == quote
                  c := byte[++tp]
                  repeat while c <> quote and c
                     dsp.out(c)
                     c := byte[++tp]
                  if c <> quote
                     abort @syn
                  if skipspaces <> ";"
                     abort @syn
                  nt := skipspaces
               if not isvar(nt)
                  abort @syn
               b := 0
               inVars[b++] := fixvar(nt)
               repeat while skipspaces == ","
                  nt := skipspaces
                  if not isvar(nt) or b == 26
                     abort @syn
                  inVars[b++] := fixvar(nt)
               getline
               tokenize
               repeat a from 1 to b
                  vars[inVars[a-1]] := expr
                  if a < b
                     if spaces == ","
                        skipspaces
            131: ' PRINT
               a := 0
               repeat
                  nt := spaces
                  if nt == 0 or nt == ":"
                     quit
                  if nt == quote
                     tp++
                     repeat
                        c := byte[tp++]
                        if c == 0 or c == quote
                           quit
                        dsp.out(c)
                        a++
                  elseif nt == "{"
                     tp++
                     d := expr
                     if spaces <> "}"
                        abort @syn
                     tp++
                     b := expr
                     b <<= (8 - d) << 2
                     a += d
                     repeat d
                        dsp.out(lookupz((b<-=4)&$F:"0".."9","A".."F"))
                  else
                     d := (b := expr) == negx
                     if b < 0
                        b := ||(b+d)
                        dsp.out("-")
                        a++
                     c := 1_000_000_000
                     d~
                     repeat 10
                        if b => c
                           dsp.out(b / c + "0")
                           a++
                           b //= c
                           d~~
                        elseif d or c == 1
                           dsp.out("0")
                           a++
                        c /= 10
                  nt := spaces
                  if nt == ";"
                     tp++
                  elseif nt == ","
                     dsp.out(" ")
                     a++
                     repeat while a & 7
                        dsp.out(" ")
                        a++
                     tp++
                  elseif nt == 0 or nt == ":"
                     dsp.out(fReturn)
                     quit
                  else
                     abort @syn
            132, 133: ' GOTO, GOSUB
               a := expr
               if a < 0 or a => 65535
                  abort @ln
               b := findline(a)
               if wordat(b) <> a
                  abort @ln
               if ht == 133
                  pushstack
               nextlineloc := b 
            134: ' RETURN
               if sp == 0
                  abort string("INVALID RETURN")
               nextlineloc := stack[--sp]
            135: ' REM
               repeat while skipspaces
            136: ' NEW
               clearall
            137: ' LIST {<expr> {,<expr>}}
               b := 0                ' Default line range
               c := 65535
               if spaces <> 0        ' At least one parameter
                  b := c := expr
                  if spaces == ","
                     skipspaces
                     c := expr
               a := long[userPtr]
               repeat while a+2 < eop
                  d := wordat(a)
                  if d => b and d =< c
                     dsp.dec(d)
                     dsp.out(" ")
                     putlinet(a+2)
                  a += 3 + strsize(a+2)
            138: ' RUN
                  clearvars
            140: ' OPEN " <file> ", R/W/A/D/X
               if spaces <> quote
                  abort @syn
               scanFilename(@f0)
               if spaces <> ","
                  abort @syn
               case skipspaces
                  "A", "a": d := "A"
                  "W", "w": d := "W"
                  "R", "r": d := "R"
                  "D", "d": d := "D"
                  "X", "x": d := "X"
                  other: abort string("Invalid open file mode")
               tp++
               if f0.byte == ":"     ' Flash file
                  ifnot spi.initFile(@fcb,@f0+1)
                     abort string("Invalid flash filename")
                  case d
                     "D": ifnot spi.eraseFile(@fcb)
                             abort string("Can't delete flash file")
                          longfill(@fcb,-1,4)
                     "R": ifnot spi.openFile(@fcb)
                             abort string("Can't open flash file")
                          fileOpened := true
                     "W": ifnot spi.createFile(@fcb)
                             abort string("Can't create flash file")
                          fileOpened := true
                     "X": ifnot spi.openFile(@fcb)
                             abort string("Can't execute flash file")
                          spi.bootFile(@fcb,true)
                     other: abort string("Invalid open file mode")
               else
                  if \drv.Open(@fileStuff,@f0,d)
                     abort string("Can't open file")
                  fileOpened := true
            141: ' READ <var> {, <var> }
               if not isvar(nt)
                  abort @syn
               d := 0
               inVars[d++] := fixvar(nt)
               repeat while skipspaces == ","
                  nt := skipspaces
                  if not isvar(nt) or d == 26
                     abort @syn
                  inVars[d++] := fixvar(nt)
               a := 0
               repeat
                  if (c := massIn) < 0
                     abort string("Can't read file")
                  elseif c == fReturn or c == fEof
                     tline[a] := 0
                     tp := @tline
                     quit
                  elseif c == fLinefeed
                     next
                  elseif a < linelen-1
                     tline[a++] := c
               tokenize
               repeat a from 1 to d
                  vars[inVars[a-1]] := expr
                  if a < d
                     if spaces == ","
                        skipspaces
            142: ' WRITE ...
               d := 0 ' record column
               repeat
                  nt := spaces
                  if nt == 0 or nt == ":"
                     quit
                  if nt == quote
                     tp++
                     repeat
                        c := byte[tp++]
                        if c == 0 or c == quote
                           quit
                        massOut(c)
                        d++
                  elseif nt == "{"
                     tp++
                     a := expr
                     if spaces <> "}"
                        abort @syn
                     tp++
                     b := expr
                     b <<= (8 - a) << 2
                     d += a
                     repeat a
                        massOut(lookupz((b<-=4)&$F:"0".."9","A".."F"))
                  else
                     a := expr
                     c := a == negx
                     if a < 0
                        a := ||(a+c)
                        massOut("-")
                        d++
                     b := 1_000_000_000
                     c := false
                     repeat 10
                        if a => b
                           massOut(a / b + "0")
                           d++
                           a //= b
                           c := true
                        elseif c or b == 1
                           massOut("0")
                           d++
                        b /= 10
                  nt := spaces
                  if nt == ";"
                     tp++
                  elseif nt == ","
                     massOut(" ")
                     d++
                     repeat while d & 7
                        massOut(" ")
                        d++
                     tp++
                  elseif nt == 0 or nt == ":"
                     massOut(fReturn)
                     massOut(fLinefeed)
                     quit
                  else
                     abort @syn
            143: ' CLOSE
               fileOpened := false
               if isFlashFile
                  longfill(@fcb,-1,4)
               else
                  \drv.Close(@fileStuff)
            144: ' DELETE " <file> "
               if spaces <> quote
                  abort @syn
               scanFilename(@f0)
               if f0.byte == ":"
                  ifnot spi.initFile(@fcb,@f0+1)
                     abort string("Invalid flash filename")
                  ifnot spi.eraseFile(@fcb)
                     abort string("Can't delete flash file")
               else
                  if \drv.Open(@fileStuff,@f0,"D")
                     abort string("Can't delete file")
                  \drv.Close(@fileStuff)
            145: ' RENAME " <file> "," <file> "
               if spaces <> quote
                  abort @syn
               scanFilename(@f0)
               if spaces <> ","
                  abort @syn
               if skipspaces <> quote
                  abort @syn
               scanFilename(@f0)
               abort string("Rename not implemented")
            146: ' FILES {" <file> "}
               if spaces == quote
                  scanFilename(@f0)
               else
                  bytemove(@f0,string("*.*"),4)
               ExpandWildcards(@f0)
               dsp.str(string("SD Card Root Directory",fReturn))
               MyMount
               PrintDir(@f0)
               spi.firstFile(@fcb)
               dsp.str(string("Flash Drive Directory",fReturn))
               c := 3
               repeat while spi.nextFile(@fcb) and Match( @fcb, @f0, 11)
                  b := 1
                  repeat a from 0 to 7
                     if fcb.byte[a] == $FF
                        b++
                     else
                        dsp.out(fcb.byte[a])
                  dsp.out(".")
                  repeat a from 8 to 10
                     if fcb.byte[a] == $FF
                        b++
                     else
                        dsp.out(fcb.byte[a])
                  if --c
                     repeat b
                        dsp.out(" ")
                  else
                     dsp.out(fReturn)
                     c := 3
               if c <> 3
                  dsp.out(fReturn)
               longfill(@fcb,-1,4)
            147: ' SAVE or SAVE "<filename>"
               if (nt := spaces) == quote
                  scanFilename(@f0)
                  if f0.byte == ":"
                     ifnot spi.initFile(@fcb,@f0+1)
                        abort string("Invalid file name format")
                     ifnot spi.createFile(@fcb)
                        abort string("Can't create flash file")
                  else
                     if \drv.Open(@fileStuff,@f0,"W")
                        abort string("Can't create file")
                  processSave
                  if isFlashFile
                     longfill(@fcb,-1,4)         ' Close FCB
                  else
                     \drv.Close(@fileStuff)
               else
                  if nt == "["                   ' Align save area for paged writes
                     a := getaddress(",") + 64
                     if (a & 63) == 63
                        a += 64
                     a := (a & $7FFC0) | (i2c#BootPin << 18)
                  else
                     a := ((userPtr - progsize - 62) & $7FC0) | (i2c#BootPin << 18)
                  nt := spaces
                  if nt <> 0 and nt <> ":"
                     abort @syn                  ' Write program to EEPROM
                  d := eop - long[userPtr] + 1
                  if i2c.writeWord(i2c#BootPin,i2c#EEPROM,a-2,d) ' Program size
                     abort string("Save EEPROM write")
                  b := cnt
                  repeat while i2c.writeWait(i2c#BootPin,i2c#EEPROM,a-2)
                     if cnt - b > clkfreq / 10
                        abort string("Save EEPROM timeout")
                  repeat c from 0 to d step 64   ' Write the program itself
                     if i2c.writePage(i2c#BootPin,i2c#EEPROM,a+c,long[userPtr]+c,d-c<#64)
                        abort string("Save EEPROM write")
                     repeat while i2c.writeWait(i2c#BootPin,i2c#EEPROM,a+c)
                        if cnt - b > clkfreq / 10
                           abort string("Save EEPROM timeout")
            148: ' LOAD or LOAD "<filename>"
               if (nt := spaces) == quote
                  scanFilename(@f0)
                  if f0.byte == ":"
                     ifnot spi.initFile(@fcb,@f0)
                        abort string("Invalid file name format")
                     ifnot spi.openFile(@fcb)
                        abort string("Can't open flash file")
                  else
                     if \drv.Open(@fileStuff,@f0,"R") ' Open requested file
                        abort string("Can't open file")
                  a := @tailLine                 ' Save statement tail
                  repeat while byte[a++] := byte[tp++]
                  newprog
                  processLoad
                  tp := @tailLine                ' Scan copy after load
                  if isFlashFile
                     longfill(@fcb,-1,4)         ' Close FCB
                  else
                     \drv.Close(@fileStuff)
               else
                  if nt == "["                   ' Align save area for paged writes
                     a := getaddress(",") + 64
                     if (a & 63) == 63
                        a += 64
                     a := (a & $7FFC0) | (i2c#BootPin << 18)
                  else
                     a := ((userPtr - progsize - 62) & $7FC0) | (i2c#BootPin << 18)
                  nt := spaces
                  if nt <> 0 and nt <> ":"
                     abort @syn                  ' Read program from EEPROM
                  if i2c.readPage(i2c#BootPin,i2c#EEPROM,a-2,@d,2)
                     abort string("Load EEPROM read")
                  d &= $FFFF
                  if d < 3 or d > progsize       ' Read program size & check
                     abort string("Invalid program size")
                  c := @tailLine                 ' Save statement tail
                  repeat while byte[c++] := byte[tp++]
                  tp := @tailLine                ' Scan copy after load
                  if i2c.readPage(i2c#BootPin,i2c#EEPROM,a,long[userPtr],d)
                     abort string("Load EEPROM read")
                  eop := long[userPtr] + d - 1
                  nextlineloc := eop - 2         ' Leave it stopped
            154: ' FOR <var> = <expr> TO <expr> {STEP <expr>}
               ht := spaces
               if ht == 0
                  abort @syn
               nt := skipspaces
               if not isvar(ht) or nt <> "="
                  abort @syn
               a := fixvar(ht)
               skipspaces
               vars[a] := expr
               if spaces <> 155 ' TO             ' Save FOR limit
                  abort @syn
               skipspaces
               forLimit[a] := expr
               if spaces == 156 ' STEP           ' Save step size
                  skipspaces
                  forStep[a] := expr
               else
                  forStep[a] := 1                ' Default step is 1
               if spaces
                  abort @syn
               forLoop[a] := nextlineloc         ' Save address of line
               if forStep[a] < 0                 '  following the FOR
                  b := vars[a] => forLimit[a]
               else                              ' Initially past the limit?
                  b := vars[a] =< forLimit[a]
               if not b                          ' Search for matching NEXT 
                  repeat while nextlineloc < eop-2
                     curlineno := wordat(nextlineloc)
                     tp := nextlineloc + 2
                     nextlineloc := tp + strsize(tp) + 1
                     if spaces == 157            ' NEXT <var>
                        nt := skipspaces         ' Variable has to agree
                        if not isvar(nt)
                           abort @syn
                        if fixvar(nt) == a       ' If match, continue after
                           quit                  '  the matching NEXT
            157: ' NEXT <var>
               nt := spaces
               if not isvar(nt)
                  abort @syn
               a := fixvar(nt)
               vars[a] += forStep[a]             ' Increment or decrement the
               if forStep[a] < 0                 '  FOR variable and check for
                  b := vars[a] => forLimit[a]
               else                              '  the limit value
                  b := vars[a] =< forLimit[a]
               if b                              ' If continuing loop, go to
                  nextlineloc := forLoop[a]      '  statement after FOR
               tp++
            159: ' OUTA [ <expr>{..<expr>} ] = <expr>
               a := getAddress(".")
               if a > $FFFF
                  b := a & $FF
                  a := (a >> 8) & $FF
                  outa[a..b] := specialExpr
                  dira[a..b]~~
                  repeat c from a to b
                     outputs |= |< c
               else
                  outa[a] := specialExpr
                  dira[a]~~
                  outputs |= |< a
            160: ' PAUSE <expr> {,<expr>}
               if pauseTime == 0                 ' If no active pause time, set it
                  spaces                         '  with a minimum time of 50us
                  pauseTime := expr * 1000
                  if spaces == ","               ' First (or only) value is in ms
                     skipspaces
                     pauseTime += expr           ' Second value is in us
                  pauseTime #>= 50
               if pauseTime < 10_050             ' Normally pause at most 10ms at a time,
                  waitcnt(pauseTime * uS + cnt)  '  but, if that would leave < 50us,
                  pauseTime := 0                 '   pause the whole amount now
               else                             
                  a := pauseTime <# 10_000     
                  waitcnt(a * uS + cnt)          ' Otherwise, pause at most 10ms and
                  nextlineloc := thisLine        '  re-execute the PAUSE for the rest
                  pauseTime -= 10_000
            166: ' BYTE [ <expr> ] = <expr>
               a := getAddress(" ")
               byte[a] := specialExpr
            167: ' WORD [ <expr> ] = <expr>
               a := getAddress(" ")
               word[a] := specialExpr
            168: ' LONG [ <expr> ] = <expr>
               a := getAddress(" ")
               long[a] := specialExpr
            170: ' PHSA =
               PHSA := specialExpr
            171: ' PHSB =
               PHSB := specialExpr
            172: ' FRQA =
               FRQA := specialExpr
            173: ' FRQB =
               FRQB := specialExpr
            174: ' CTRA =
               CTRA := specialExpr
            175: ' CTRB =
               CTRB := specialExpr
            176: ' DISPLAY <expr> {,<expr>}
               spaces
               dsp.out(expr)
               repeat while spaces == ","
                  skipspaces
                  dsp.out(expr)
            178: ' LET <var> = <expr>
               nt := spaces
               if not isvar(nt)
                  abort @syn
               tp++
               vars[fixvar(nt)] := specialExpr
            179: ' STOP
               nextlineloc := eop-2
               return
            180: ' END
               nextlineloc := eop-2
               return
            181: ' EEPROM [ <expr> ] = <expr>
               a := getAddress(",")
               if i2c.writeByte(i2c#BootPin,i2c#EEPROM,a,specialExpr)
                  abort string("EEPROM write")
               b := cnt                            ' Wait for write to finish
               repeat while i2c.writeWait(i2c#BootPin,i2c#EEPROM,a)
                  if cnt - b > clkfreq / 10        ' Maximum 100ms
                     abort string("EEPROM timeout")
            182: ' FILE = <expr>
               massOut(specialExpr)
            184: ' SPIN [{<expr>,}<expr>] or "<file>"
               if spaces == quote
                  scanFilename(@f0)
                  if f0.byte == ":"
                     ifnot spi.initFile(@fcb,@f0+1)
                        abort string("Invalid flash filename")
                     ifnot spi.openFile(@fcb)
                        abort string("Can't execute flash file")
                     spi.bootFile(@fcb,true)
                  else
                     if \drv.Open(@fileStuff,@f0,"X") ' Open requested file
                        abort string("Can't execute file")
                     \drv.Close(@fileStuff)
                  abort string("Can't execute file")
               else
                  a := getAddress(",") & !$7FFF
                  if i2c.readByte(i2c#BootPin,i2c#EEPROM,a) == -1
                     abort string("No EEPROM there")
                  if a == (i2c#BootPin << 18)    ' Execute 1st 32K of EEPROM
                     reboot
                  else
                     abort string("Can't execute EEPROM program")
            185: ' COPY [<expr>],"<file>" or COPY "<file>",[<expr>] or
                 ' COPY [<expr>],[<expr>] where <expr> are different
               if spaces == quote
                  scanFileName(@f0)
                  if spaces <> ","
                     abort @syn
                  skipspaces
                  b := getAddress(",") & !$7FFF
                  if i2c.readByte(i2c#BootPin,i2c#EEPROM,a) == -1
                     abort string("No EEPROM there")
                  if f0.byte == ":"
                     ifnot spi.initFile(@fcb,@f0+1)
                        abort string("Invalid flash filename")
                     ifnot spi.openFile(@fcb)
                        abort string("Can't open flash file")
                  else
                     if \drv.Open(@fileStuff,@f0,"R")
                        abort string("Can't open file")
                     longfill(@fcb,-1,4)
                  if isFlashFile
                     ifnot spi.readFile(@fcb,@f0,32)
                        abort string("Can't read program")
                  else
                     if \drv.Read(@fileStuff,@f0,32) <> d
                        abort string("Can't read program")
                  if i2c.writePage(i2c#BootPin,i2c#EEPROM,b,@f0,32)
                     abort string("Copy EEPROM write error")
                  c := cnt
                  repeat while i2c.writeWait(i2c#BootPin,i2c#EEPROM,b)
                     if cnt - c > clkfreq / 10
                        abort string("Copy EEPROM wait error")
                  a := word[@f0+vbase]
                  repeat c from 32 to a - 1 step 32
                     d := (a - c) <# 32
                     if isFlashFile
                        ifnot spi.readFile(@fcb,@f0,d)
                           abort string("Can't read program")
                     else
                        if \drv.Read(@fileStuff,@f0,d) <> d
                           abort string("Can't read program")
                     if i2c.writePage(i2c#BootPin,i2c#EEPROM,b+c,@f0,d)
                        abort string("Copy EEPROM write error")
                     e := cnt
                     repeat while i2c.writeWait(i2c#BootPin,i2c#EEPROM,b+c)
                        if cnt - e > clkfreq / 10
                           abort string("Copy EEPROM wait error")
                  if isFlashFile
                     longfill(@fcb,-1,4)         ' Close flash file
                  else
                     \drv.Close(@fileStuff)
               else
                  a := getAddress(",") & !$7FFF
                  if i2c.readByte(i2c#BootPin,i2c#EEPROM,a) == -1
                     abort string("No EEPROM there")
                  if spaces <> ","
                     abort @syn
                  skipspaces
                  if spaces == quote
                     scanFileName(@f0)
                     if f0.byte == ":"
                        ifnot spi.initFile(@fcb,@f0+1)
                           abort string("Invalid flash filename")
                        ifnot spi.createFile(@fcb)
                           abort string("Can't create flash file")
                     else
                        if \drv.Open(@fileStuff,@f0,"W")
                           abort string("Can't create file")
                        longfill(@fcb,-1,4)
                     b := 0
                     if i2c.readPage(i2c#BootPin,i2c#EEPROM,a+vbase,@b,2)
                        abort string("Copy EEPROM read error")
                     repeat c from 0 to b - 1 step 32
                        d := (b - c) <# 32
                        if i2c.readPage(i2c#BootPin,i2c#EEPROM,a+c,@f0,d)
                           abort string("Copy EEPROM read error")
                        if isFlashFile
                           ifnot spi.writeFile(@fcb,@f0,d)
                              abort string("Can't save program")
                        else
                           if \drv.Write(@fileStuff,@f0,d) <> d
                              abort string("Can't save program")
                     ifnot isFlashFile
                        \drv.Close(@fileStuff)
                  else
                     if a == (b := getAddress(",") & !$7FFF)
                        abort string("EEPROM areas same")
                     if i2c.readByte(i2c#BootPin,i2c#EEPROM,a) == -1
                        abort string("No EEPROM there")
                     d := 0
                     if i2c.readPage(i2c#BootPin,i2c#EEPROM,a+vbase,@d,2)
                        abort string("Copy EEPROM read error")
                     repeat c from 0 to d - 1 step 32
                        if i2c.readPage(i2c#BootPin,i2c#EEPROM,a+c,@f0,32)
                           abort string("Copy EEPROM read error")
                        if i2c.writePage(i2c#BootPin,i2c#EEPROM,b+c,@f0,32)
                           abort string("Copy EEPROM write error")
                        e := cnt
                        repeat while i2c.writeWait(i2c#BootPin,i2c#EEPROM,b+c)
                           if cnt - e > clkfreq / 10
                              abort string("Copy EEPROM wait error")
            186: ' DUMP <expr>,<expr> or DUMP [<expr>],<expr>
               if spaces == "["
                  c := getAddress(",")
                  a := c & $F80000
                  b := c & $07FFFF
               else
                  a := -1
                  b := expr
               if spaces <> ","
                  abort @syn
               skipspaces
               dumpMemory(a,b,expr)
            187: ' SRAM [ <expr> {,<expr>} ] = <expr>
               if spaces <> "["
                  abort @syn
               skipspaces
               a := expr
               if spaces == ","
                  skipspaces
                  b := expr
               else
                  b := 1
               if spaces <> "]" or b < 1 or b > 4
                  abort @syn
               skipspaces
               spi.writeSRAM(a,specialExpr,-b)
            188: ' ERASE <expr>
               spi.eraseData(expr)
            189: ' FLASH [ <expr> {,<expr>} ] = <expr>
               if spaces <> "["
                  abort @syn
               skipspaces
               a := expr
               if spaces == ","
                  skipspaces
                  b := expr
               else
                  b := 1
               if spaces <> "]" or b < 1 or b > 4
                  abort @syn
               skipspaces
               spi.writeData(a,specialExpr,-b)
            191: ' FCB [ <expr> ] = <expr>
               a := getAddress(" ")
               b := specialExpr
               if a < 0 or a > 3
                  abort @syn
               fcb[a] := b
            192: ' EXIT
               reboot              ' Sphinx may not be active, so just reboot
      else                                   
         abort(@syn)
      if spaces == ":"
         restart := 1
         tp++

pri doline(s) | c                 ' Execute the string in s or wait for input
   curlineno := -1
   if key.peekKey == breakKey     ' Was the "break key" pressed?
      repeat while key.key        ' If so, clear the keyboard buffer
      nextlineloc := eop-2        ' Stop the program
   if nextlineloc < eop-2
      curlineno := wordat(nextlineloc)
      tp := nextlineloc + 2
      nextlineloc := tp + strsize(tp) + 1
      texec
   else
      if fileOpened
         fileOpened := false
         \drv.Close(@fileStuff)
      pauseTime := 0
      repeat c from 0 to 15
         if outputs & |< c
            dira[c]~
            outa[c]~
      outputs := 0
      if s
         bytemove(tp:=@tline,s,strsize(s)+1)
      else
         putlinet(string("OK"))
         getline
      c := spaces
      if "0" =< c and c =< "9"
         insertline
         nextlineloc := eop - 2
      else
         tokenize
         if spaces
            texec

PRI processLoad : c | a
   repeat
      a := 0
      repeat
         if (c := massIn) < 0
            quit
         if c == fReturn or c == fEof
            tline[a] := 0
            tp := @tline
            quit
         elseif c == fLinefeed
            next
         elseif a < linelen-1
            tline[a++] := c
      if c == fEof and tline[0] == 0
         quit
      if c < 0
         abort string("Error while loading file")
      tp := @tline
      a := spaces
      if "0" =< a and a =< "9"
         insertline
         nextlineloc := eop - 2
      else
         if a <> 0
            abort string("Missing line number in file")

PRI processSave | a, c, d, ntoks
   ntoks := (@tokx - @toks) / 2
   a := long[userPtr]
   repeat while a+2 < eop
      d := wordat(a)
      massDec(d)
      massOut(" ")
      d := a + 2
      repeat while c := byte[d++]
         if c => 128
            if (c -= 128) < ntoks
               massOut(@@toks[c])
               if c <> 7   ' REM
                  massOut(" ")
            else
               massOut("{")
               massDec(c)
               massOut("}")
         else
            massOut(c)
      massOut(fReturn)
      massOut(fLinefeed)
      a += 3 + strsize(a+2)

PRI getAnyNumber | c, t
   case c := byte[tp]
      quote:
         if result := byte[++tp]
            if byte[++tp] == quote
              tp++
            else
               abort string("missing closing quote")
         else
            abort string("end of line in string")
      "$":
         c := byte[++tp]
         if (t := hexDigit(c)) < 0
            abort string("invalid hex character")
         result := t
         c := byte[++tp]
         repeat until (t := hexDigit(c)) < 0
            result := result << 4 | t
            c := byte[++tp]
      "%":
         c := byte[++tp]
         if not (c == "0" or c == "1")
            abort string("invalid binary character")
         result := c - "0"
         c := byte[++tp]
         repeat while c == "0" or c == "1"
            result := result << 1 | (c - "0")
            c := byte[++tp]
      "0".."9":
         result := c - "0"
         c := byte[++tp]
         repeat while c => "0" and c =< "9"
            result := result * 10 + c - "0"
            c := byte[++tp]
      other:
        abort string("invalid literal value")

PRI hexDigit(c)
'' Convert hexadecimal character to the corresponding value or -1 if invalid.
   if c => "0" and c =< "9"
      return c - "0"
   if c => "A" and c =< "F"
      return c - "A" + 10
   if c => "a" and c =< "f"
      return c - "a" + 10
   return -1

PUB dumpMemory(pin,addr,size) | i, c, p, first, buf0, buf1, buf2
'' This routine dumps a portion of the RAM/ROM to the display (pin == -1).
'' If pin is not -1, it is an EEPROM address in the form required by the
'' I2C routines in i2cSpiInit.  The specified address is or'd with this.
'' The format is 8 bytes wide with hexadecimal and ASCII.
   addr &= $7FFFF
   first := true
   p := addr & $7FFF8
   repeat while p < (addr + size)
      if first
         dsp.hex(addr,5)
         first := false
      else
         dsp.hex(p,5)
      dsp.out(":")
      repeat i from 0 to 7
         byte[@buf0][i] := " "
         if p => addr and p < (addr + size)
            if pin <> -1
               c := 0
               if i2c.readPage(i2c#BootPin,i2c#EEPROM,pin|p,@c,1)
                  abort string("EEPROM read")
            else
               c := byte[p]
            dsp.hex(c,2)
            if c => " " and c =< "~"
               byte[@buf0][i] := c
         else
            dsp.out(" ")
            dsp.out(" ")
         dsp.out(" ")
         p++
      buf2 := 0
      dsp.out("|")
      dsp.str(@buf0)
      dsp.out("|")
      dsp.out(fReturn)

con
pCommand = SDSPIRENDEZVOUS
pParam = pCommand + 4
pBlockno = pCommand + 8

pri readblock(n, b)
'
'   Read a single block.  The "n" passed in is the
'   block number (blocks are 512 bytes); the b passed
'   in is the address of 512 blocks to fill with the
'   data.
'                              
   long[pParam] := b
   long[pBlockno] := n
   long[pCommand] := "R"
   repeat while long[pCommand]
   if long[pParam]
      abort long[pParam]
   return 0

con
   SECTORSIZE = 512
   SECTORSHIFT = 9
   DIRSIZE = 32
   DIRSHIFT = 5

{ long }      _length       = 0
{ long }      _leftInFile   = 4
{ long }      _sopDir       = 8  ' sector/offset "pointer" to directory entry
{ long }      _sopTail      = 12 ' sector/offset "pointer" to last cluster# in FAT chain
{ word }      _cluster      = 16
{ word }      _sector       = 18 ' sector# within cluster; ranges from 0 to sectorsPerCluster-1
{ word }      _bp           = 20 ' byte pointer (actually an index into the sector buffer); range [0..511]
              _buffer       = 24

SIZEOFIOBLOCK = SECTORSIZE + 24

dat
                        long
myMetadataBuffer        byte    0[512]
myMetadataSector        word    -1
dirty                   byte    0
clusterShift            byte    0

pri ReadMetadataSector( s )
  if myMetadataSector == s
    return
  readblock( myMetadataSector := s, @myMetadataBuffer )

pri ExpandWildcards(ptr) | i, j, f0,f1,f2,f3
  bytemove( @f0, ptr, strsize(ptr)+1 ) ' copy search string to local buffer
  i~
  j~
  bytefill( ptr, " ", 11 )             ' zero terminated 8+3=11 byte string
  byte[ptr+11]~

  repeat while f0.byte[j]
    if f0.byte[j] == "*"
      if i < 8
        repeat
          byte[ptr][i++] := "?"
        until i == 8
      elseif i < 11
        repeat
          byte[ptr][i++] := "?"
        until i == 11
      else
        abort string("bad wildcard")
    elseif f0.byte[j] == "."
      if i =< 8
        i := 8
      else
        abort string("filename too long")
    else
      if i < 11
        byte[ptr][i++] := f0.byte[j]
      else
        abort string("filename too long")
    ++j

pri PrintDir(ptr) | s, p, c, b, d, pad
  b := 0
  d := false
  s := dirSector0
  repeat numberOfRootDirectoryEntries >> 4
    p := @myMetadataBuffer
    repeat 16
      ReadMetadataSector( s )
      ifnot byte[p]
        if d                           ' Finish up display
          dsp.out( 13 )
        return
      if byte[p] == $e5                ' Erased directory entry
        p += 32
        next
      if byte[p][$0b] & $0f == 0 and Match( p, ptr, 11 )
        d := true
        if b == 39                     ' 3 directory entries per line
          dsp.out( 13 )
          b := 0
        pad := 1
        repeat 8                       ' display file name
          result := byte[p++]
          if result == " "             ' blank suppress
            pad++
          else
            dsp.out( result )
        dsp.out( "." )                 ' display "."
        repeat 3                       ' display extension
          result := byte[p++]
          if result == " "             ' blank suppress
            pad++
          else
            dsp.out( result )
        p -= 11
        repeat pad                     ' pad out suppressed blanks
          dsp.out( " " )
        b += 13
      p += 32
    ++s

pri Match( p, q, n ) | c1, c2
  repeat n
    c1 := byte[p++]
    c2 := byte[q++]
    if "a" =< c1 and c1 =< "z"
      c1 += "A" - "a"
    if "a" =< c2 and c2 =< "z"
      c2 += "A" - "a"
    if c1 == $FF and c2 == " "         ' special case for flash directory
      next
    if c2 == "?"
      next
    if c1 <> c2
      return false
  return true

pri MyMount | pbrSector  ,i
  pbrSector~
  repeat
    ReadMetadataSector( pbrSector )
    if bytecomp( @myMetadataBuffer+$36, string("FAT16"), 5 )
      quit
    if pbrSector
      abort -20                 ' not FAT16
    pbrSector := PeekL( @myMetadataBuffer + $1c6 )

  bytesPerSector := PeekW( @myMetadataBuffer + $0b )
  sectorsPerCluster := myMetadataBuffer[$0d]
  clusterShift := >| sectorsPerCluster - 1
  reservedSectors := PeekW( @myMetadataBuffer + $0e )
  numberOfFats := myMetadataBuffer[$10]
  sectorsPerFat := PeekW( @myMetadataBuffer + $16 )
  numberOfRootDirectoryEntries := PeekW( @myMetadataBuffer + $11 )
  fatSector0 := pbrSector + reservedSectors
  dirSector0 := fatSector0 + numberOfFats * sectorsPerFat
  dataSector0 := dirSector0 + numberOfRootDirectoryEntries >> 4

pri bytecomp( p, q, n )
  repeat n
    if byte[p++] <> byte[q++]
      return
  return true
  
pri PeekW( a )
  return byte[a++] + byte[a] << 8

pri PeekL( a )
  return byte[a++] + byte[a++] << 8 + byte[a++] << 16 + byte[a] << 24

{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
