''****************************
''*  Chicken for Propeller   *
''*  Doug Dingus & JT Cook   *
''****************************
''
'' Controls - use keyboard or gamepad to play
'' Start Game
''  Hit the Spacebar or B button on the gamepad
'' Left chicken
''  Q - move up, A - move down
''  Gamepad 1 up and down
'' Right chicken
''  Up Arrow key - move up, Down Arrow key - move down
''  Gamepad 2 up and down
''
'' Modifications by JT Cook
'' 06-20-10 - changed controller input, corrected game start/stop routine, reset score
''            after timer expired, changed how timer worked
'' 09-24-10 - re-org'd a lot of the code, changed cars into types, filled all lanes
''           added collision detection, made chicken fall to bottom when hit by a car
'' 10-08-10 - added horn sound, fixed chicken graphic
'' 10-23-10 - fixed color on large car so it wouldn't mess up sync on certain TVs
'' 10-30-10 - added keyboard controls
'' 11-06-10 - added engine and horn sounds
'' 11-07-10 - added sound effect for cars passing by chicken

'sound setup
'channels 0, 1- horn
'channel 2 - falling sound
'channel 3 - score add sound
'channel 4 - hit by car
'channel 5 - engine sound
{

SPRITE NOTES

8x8 Sprites numbered from 0 - 255, indexed by SPRC

sptr allocation = one long per sprite

0-1   =   Chicken 0  SPRC 114, 116 ; SPRP 1
1-2   =   Chicken 1


Cars start at sprite 3 (sptr + 16) ; 64 Sprites Total Avaliable

Type 1 Small  =  2 Sprites, sptr + 8 ; SPRP 1 ; Start --> SPRC 96, 128  (8 cars total)

Type 2 Medium =  4 Sprites, sptr + 16 ; SPRP 1 ; Start --> SPRC 104, 105, 136, 137  (5 cars total)

Type 3 Large  =  8 Sprites, sptr + 32 ; SPRP 1 ; Start --> SPRC 160 - 163, 192 - 195 (3 cars total)


Upper Lanes Start Scan 24 ; 5 Lanes  Increasing

Lower Lanes Start Scan 104 ; 5 Lanes Increasing


TODO:  Random Car Field
          Direction, Type, Lane, Number Of
       Troubleshoot Palette Problem with Big Cars
       Pick way better colors :(
       Troubleshoot both chickens moving at once
       
Sample data renders at 2 cogs...

}

CON    'Driver constants
 _stack = (100) >> 2   'accomodate display memory and stack
{ Hybrid Settings
_clkmode = xtal1 + pll16x
_xinfreq = 6_000_000

modepins=%011_0000
}

{ Proto/Demoboard Settings
_clkmode = xtal1 + pll16x
_xinfreq = 6_000_000

modepins=%001_0101
}

{ Hydra Settings
_clkmode = xtal1 + pll8x
_xinfreq = 10_500_000
_stack = (100) >> 2   'accomodate display memory and stack
modepins=%011_0000
}

'{ C3 Settings
  _clkmode = xtal1 + pll16x  '
  _xinfreq = 5_000_000 + 0_000 
  modepins=%001_0101
'}
  
PAL=%0000
NTSC=%0001
INTERLACED=%0000

Game_Timer = (60*(60*3)) '3 minutes

fall_value = 24 ' how many pixels the chicken will drop when falling

COGS = 2

x_tiles = 10 '*16=160
y_tiles = 12 '*16=192
mode = 0
myhx = 17+(mode&1)*4

SPRX = 0
SPRY = 1
SPRP = 2
SPRC = 3

CON     'Constants required for game

OVER =       %0000_0000
INIT =       %0000_0001
ATTRACT =    %0000_0010
STOP =       %0000_0100
RUN =        %0000_1000
engine_vol_m_0 = 255  'volume for car engine when in game

total_cars = 10
logo_animation = 0 '1 on, 0 off

''speed of the cars
speed1 = 1
speed2 = 2
speed3 = 3


OBJ
  tv    : "JB_tv_030"
  rend  : "JB_drv_030"
'  joystick : "gamepad_drv_001_hybrid.spin"
  joystick : "c3_gamepad_keyboard_v001a.spin"
  snd   : "ns_sound_drv_052_22khz_16bit.spin"

VAR     'Core working variables required for driver to operate
  long  frame                   'Frame counter
  long  nframe                  'Negative frame counter 
  long  rnd
  long  scrptr[2]               'screen buffer pointer
  byte  scrrendbuf[COGS*256]
  long  sptr
  byte player_fall[2] 'if set to 1, player will fall to the ground(is hit)
  word carx[total_cars] 'location of cars on screen
  byte cary[total_cars]
  byte car_speed[total_cars]'
  byte car_type[total_cars] ' 0-small (8x16),1-small, 2-medium(16x16), 3-large ( 32x16 )
  byte car_dir[total_cars] '1 left, 0 right
  byte engine_sound_timer 'how long until we get to the next pitch
  byte engine_sound_value 'current location in sound map
  byte engine_sound_next 'how many ticks until we change the pitch
  long rand 'seed for randomizer
  long horn_timer 'how long until next horn sounds
  long engine_clock 'how long until a darting around sound plays
VAR     'Game specific ones
  
    
PUB Main | i, c, temp, temp2

'-----------------------------------------------------------
'  Run Once Tasks Go Here
'-----------------------------------------------------------
      gstate := INIT                'Do nothing but draw background screen
      scanbufptr:=@scrrendbuf       'Point to screen data
      rend.start(COGS,@tvparams)    'launch render cogs
      tv.start(@tvparams)           'launch video cog
      joystick.start(26,1)          'launch I/O cog
      'gstate := RUN 'debug
      'neither player is falling
       player_fall[0]:=0
       player_fall[1]:=0
      'Start sound driver
      snd.start(24)
'      byte[@spritesbuf][146]:=$FF
'      byte[@spritesbuf][147]:=$FF
'fix chicken sprite      
       byte[@SpriteSet+(147<<5)+8]:=$A0
       byte[@SpriteSet+(147<<5)+9]:=$0A
       byte[@SpriteSet+(147<<5)+10]:=$00
       byte[@SpriteSet+(147<<5)+11]:=$00                     

       byte[@SpriteSet+(147<<5)+12]:=$00
       byte[@SpriteSet+(147<<5)+13]:=$AA
       byte[@SpriteSet+(147<<5)+14]:=$00
       byte[@SpriteSet+(147<<5)+15]:=$00                     
'fix palette on big black car
        byte[@SpritePal+19]:=$02 ' car body '$7C, $7B
'        byte[@SpritePal+17]:=$FA ' car windshield        
'-----------------
' Initialize cars
'-----------------
     ''--- top lanes
     carx[0]:=frame/3
     cary[0]:=24
     car_speed[0]:=speed1
     car_dir[0]:=1
     car_type[0]:=0
     
     carx[1]:=nframe/2
     cary[1]:=40
     car_speed[1]:=speed3
     car_dir[1]:=1     
     car_type[1]:=1
     
     carx[2]:=nframe
     cary[2]:=56
     car_speed[2]:=speed2
     car_dir[2]:=1     
     car_type[2]:=2
     
     carx[3]:=75
     cary[3]:=72
     car_speed[3]:=speed1
     car_dir[3]:=1     
     car_type[3]:=3
     
     carx[4]:=nframe + 100
     cary[4]:=88
     car_speed[4]:=speed2
     car_dir[4]:=1     
     car_type[4]:=0
     ''--- bottom lanes
     carx[5]:=40
     cary[5]:=104
     car_speed[5]:=speed2
     car_dir[5]:=0     
     car_type[5]:=2

     carx[6]:=40
     cary[6]:=120
     car_speed[6]:=speed3
     car_dir[6]:=0     
     car_type[6]:=1

     carx[7]:=40
     cary[7]:=136
     car_speed[7]:=speed1
     car_dir[7]:=0     
     car_type[7]:=3
     
     carx[8]:=100
     cary[8]:=152
     car_speed[8]:=speed2
     car_dir[8]:=0     
     car_type[8]:=0
     
     carx[9]:=0
     cary[9]:=168
     car_speed[9]:=speed1
     car_dir[9]:=0     
     car_type[9]:=1
     ''mask all car X values
     repeat i from 0 to 9
       carx[i]&=$1FF
  rand:=0
  horn_timer:=60*3
  engine_sound_timer:=0 'how long until we get to the next pitch
  engine_sound_value:=0 'current location in sound map
  engine_sound_next:=4 'how many ticks until we change the pitch  
  engine_clock:=0
  snd.PlaySoundFM(5, snd#SHAPE_NOISE, 300,CONSTANT(snd#DURATION_INFINITE),engine_vol_m_0 , $2457_9DEF)  
'  snd.SetFreq(5, 100) 'change sound freq for car

'------------------------------------------------------------
'             Run Game Loop
'------------------------------------------------------------

  repeat                             'One loop per TV frame
    VSync                            'Lock Game logic to TV Frame

'------------------------------------------------------------
'             Init State
'------------------------------------------------------------
    if gstate == INIT
       game_time := Game_Timer   'set game timer
       frame:= 0          'Init Frame Counters
       nframe:= 0
       chick0y := 178     'Put Chickens on screen in start position
       chick1y := 178
       gstate := ATTRACT  'Hand off to Attract Mode
       snd.SetVolume(5,1)  'mute engine sound when not in a game
'------------------------------------------------------------
'             ATTRACT State
'------------------------------------------------------------
    if gstate == ATTRACT

       longmove(@sprites,@spritesbuf,sprite_count)         'Copy Sprite buffer to active sprite table
       sptr:=@spritesbuf                                   'Init Sprite buffer pointer

       joysticks := joystick.read_gamepad

       Do_Chicken0(40, chick0y)     'sprites 0 & 1
       Do_Chicken1(112,chick1y)   'sprites 1 & 2

       sptr:=@spritesbuf + 16   'Start of dynamic sprites; depending on cars
       rand+=1 ''seed randomizer    
       Draw_Cars_Test
       if joysticks & $F0           'If down, add to chicken position
          Reset_Score
          gstate := RUN
          snd.SetVolume(5,engine_vol_m_0) 'restore engine sound
'------------------------------------------------------------
'             RUN State State
'------------------------------------------------------------
    if gstate == RUN
       ''handle horn
       if(horn_timer==0)
        horn_timer:=((?rand & 7)+1)*30
        Honk_horn((?rand & 3))
       else
        horn_timer--
       do_engine_sound 'handle engine sound

       joysticks := joystick.read_gamepad

       longmove(@sprites,@spritesbuf,sprite_count)         'Copy Sprite buffer to active sprite table
       sptr:=@spritesbuf                                   'Init Sprite buffer pointer       
       Do_Chicken0(40, chick0y)     'sprites 0 & 1
       Do_Chicken1(112,chick1y)   'sprites 1 & 2

       sptr:=@spritesbuf + 16   'Start of dynamic sprites; depending on cars

       Draw_Cars_Test

       game_time-=1
       if game_time < 1
'          gstate := OVER
          gstate := INIT

          '--------------------------------------------------------
          '     Move Chickens
          '--------------------------------------------------------
      temp := joysticks & $FF     'Mask off the lower game pad state bits
      {
      if joysticks == 2           'If down, add to chicken position
        chick0y := chick0y + 1
      if joysticks == 1           'If up, subtract
        chick0y := chick0y - 1
      }
      'debug
'      if temp & $F0
'       Honk_horn((?rand & 3))
'        engine_clock:=1 'dashing sound
      'chicken is not falling
      if player_fall[0]==0
        if temp & 4              'If down, add to chicken position
         chick0y := chick0y + 1
        if temp & 8              'If up, subtract
         chick0y := chick0y - 1
      else  'chicken is hit by car and falling
         chick0y +=2
         snd.PlaySoundFM(2, snd#SHAPE_SINE, 800, 500,200, $2457_9DEF) 'falling
         player_fall[0]--
         
      temp2 := (joysticks & $FF00) >>8
      {
      if joysticks == 2           'If down, add to chicken position
        chick1y := chick1y + 1
      if joysticks == 1           'If up, subtract
        chick1y := chick1y - 1
      }
      if player_fall[1]==0
        if temp2 & $4               'If down, add to chicken position
          chick1y := chick1y + 1
        if temp2 & $8               'If up, subtract
          chick1y := chick1y - 1
      else  'chicken is hit by car and falling
         chick1y +=2
         snd.PlaySoundFM(2, snd#SHAPE_SINE, 800, 500,200, $2457_9DEF) 'falling
         player_fall[1]--      
      '---------------------------------------------------------
    '      Check Chickens
    '---------------------------------------------------------
      Do_Collisions
      if chick0y == 10
        Update_Score0
        chick0y := 178
        snd.PlaySoundFM(3, snd#SHAPE_SQUARE, 100, 10000,255, $2457_9DEF) 'score        
      if chick1y == 10
        Update_Score1
        chick1y := 178
        snd.PlaySoundFM(3, snd#SHAPE_SQUARE, 100, 10000,255, $2457_9DEF) 'score        
      if chick0y > 178
        player_fall[0]:=0
        chick0y := 178
      if chick1y >178
        player_fall[1]:=0
        chick1y := 178
        
    if gstate == OVER
        game_time := 400   'SB 2:20 after testing
        snd.SetVolume(5,1)  'mute engine sound when not in a game
        engine_sound_value:=0 'reset which engine note we use        
    Do_Copyright_Anim  
PUB do_engine_sound | sound_pitch'handle engine sound
    if(engine_clock>0)
{    
     '1020-600 = 420  (7 for 60)   (10 for 40) (12 for 35) (14 for 30)
      sound_pitch:=((12*(35-engine_sound_timer))+600)>>1
      snd.SetFreq(5,sound_pitch)
      engine_sound_timer++             
      if(engine_sound_timer>35)
        engine_clock:=0
}
    
     '1020-700 = 320 (9 for 35)
      sound_pitch:=((9*(35-engine_sound_timer))+700)>>1
      if(sound_pitch<300) OR (sound_pitch>2000)
         sound_pitch:=300
      snd.SetFreq(5,sound_pitch)
      engine_sound_timer++             
      if(engine_sound_timer>35)
        engine_clock:=0

    else              
        sound_pitch:=7' standard sound
        snd.SetFreq(5,(((sound_pitch*60)+300)>>1))          
        engine_sound_timer:=1
        engine_sound_value:=28         
PUB  Draw_Cars_Test | n, c, temp_x
       repeat n from 0 to 9  'run through all the cars
        carx[n]:=carx[n]|$FE00
        if car_dir[n]==0
          carx[n]+=car_speed[n]
        else
          carx[n]-=car_speed[n]        
        carx[n]&=$1FF
        
        temp_x:=((carx[n])>>1)&$FF

        case c := car_type[n]
         0:
          ' Small Red Car
          '8x10 -- + 3 Y pixels from Y location
          byte[sptr][SPRX]:=temp_x
          byte[sptr][SPRY]:=cary[n]
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=98
          sptr+=4
          byte[sptr][SPRX]:=temp_x
          byte[sptr][SPRY]:=cary[n]+8
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=130
          sptr+=4
         1:
          ' Small Blue Car
          '8x10 -- + 3 Y pixels from Y location
          byte[sptr][SPRX]:=temp_x
          byte[sptr][SPRY]:=cary[n]
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=99
          sptr+=4
          byte[sptr][SPRX]:=temp_x
          byte[sptr][SPRY]:=cary[n]+8
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=131
          sptr+=4
         2:
       ' Medium Blue Car
         '16x10 -- + 3 Y pixels from Y location
          byte[sptr][SPRX]:=temp_x
          byte[sptr][SPRY]:=cary[n]
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=104
          sptr+=4
          byte[sptr][SPRX]:=temp_x+8
          byte[sptr][SPRY]:=cary[n]
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=105
          sptr+=4
          byte[sptr][SPRX]:=temp_x                'Sprite screen position
          byte[sptr][SPRY]:=cary[n]+8              'Sprite screen position
          byte[sptr][SPRP]:=1                              'Sprite palette from file, or...
          byte[sptr][SPRC]:=136                             'Sprite definition from file
          sptr+=4
          byte[sptr][SPRX]:=temp_x+8
          byte[sptr][SPRY]:=cary[n]+8
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=137
          sptr+=4
         3:
          ' Large Black Car **Some TVs have sync problems with this car
          ' 32x10 -- + 3 Y pixels from Y location
          byte[sptr][SPRX]:=temp_x
          byte[sptr][SPRY]:=cary[n]
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=160
          sptr+=4
          byte[sptr][SPRX]:=temp_x+8
          byte[sptr][SPRY]:=cary[n]
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=161
          sptr+=4
          byte[sptr][SPRX]:=temp_x+16                      'Sprite screen position
          byte[sptr][SPRY]:=cary[n]                        'Sprite screen position
          byte[sptr][SPRP]:=1                              'Sprite palette from file, or...
          byte[sptr][SPRC]:=162                             'Sprite definition from file
          sptr+=4
          byte[sptr][SPRX]:=temp_x+24
          byte[sptr][SPRY]:=cary[n]
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=163
          sptr+=4
          
          byte[sptr][SPRX]:=temp_x
          byte[sptr][SPRY]:=cary[n]+8
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=192
          sptr+=4
          byte[sptr][SPRX]:=temp_x+8
          byte[sptr][SPRY]:=cary[n]+8
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=193
          sptr+=4
          byte[sptr][SPRX]:=temp_x+16                      'Sprite screen position
          byte[sptr][SPRY]:=cary[n]+8                      'Sprite screen position
          byte[sptr][SPRP]:=1                              'Sprite palette from file, or...
          byte[sptr][SPRC]:=194                            'Sprite definition from file
          sptr+=4
          byte[sptr][SPRX]:=temp_x+24
          byte[sptr][SPRY]:=cary[n]+8
          byte[sptr][SPRP]:=1
          byte[sptr][SPRC]:=195
          sptr+=4
         
          



PUB Do_Collisions | tempy1, tempy2, c, car_size   
'' chicken 4x6 (clipping sides of chicken -- +1 X, +2 Y)
'' cars -- top of the car is + 3 pixels from X location, 10 pixels in height
'' first car is at Y-24
'' grab top and bottom of chicken
   ''check player 1 x=40
   if(chick0y>16)
      ''check against hitting the chicken   
      tempy1:=(chick0y-16)>>4 'chicken
      ''check chicken
      if(tempy1<10)
       case c := car_type[tempy1]
         0: 'small
          car_size:=8
         1: 'small
          car_size:=8
         2: 'medium
          car_size:=16
         3: 'large
          car_size:=32
       if((carx[tempy1]>>1)+car_size =>41 AND (carx[tempy1]>>1)=<45)
           player_fall[0]:=fall_value
           snd.PlaySoundFM(4, snd#SHAPE_NOISE, 200, CONSTANT(snd#SAMPLE_RATE/7),255,$2457_9DEF)
      ''check for sound effect of car coming      
      tempy1:=(chick0y-16-8)>>4 'top of chicken
      ''check top of chicken
      if(engine_clock==0 AND tempy1<10)
       case c := car_type[tempy1]
         0: 'small
          car_size:=8
         1: 'small
          car_size:=8
         2: 'medium
          car_size:=16
         3: 'large
          car_size:=32
''       if(chick
'       if((carx[tempy1]>>1)+car_size =>41 AND (carx[tempy1]>>1)=<45)
      if(tempy1<5) ''top half of road
       if((carx[tempy1]>>1)+car_size =>45 AND (carx[tempy1]>>1)=<50)
         engine_clock:=1 'dashing sound
      else
        if((carx[tempy1]>>1)+car_size =>36 AND (carx[tempy1]>>1)=<41)
         engine_clock:=1 'dashing sound                                     
         
      tempy1:=(chick0y-16+8)>>4 'bottom of chicken
      ''check top of chicken
      if(engine_clock==0 AND tempy1<10)
       case c := car_type[tempy1]
         0: 'small
          car_size:=8
         1: 'small
          car_size:=8
         2: 'medium
          car_size:=16
         3: 'large
          car_size:=32
''       if(chick
       if(tempy1<5) ''top half of road
'       if((carx[tempy1]>>1)+car_size =>41 AND (carx[tempy1]>>1)=<45)
       if((carx[tempy1]>>1)+car_size =>45 AND (carx[tempy1]>>1)=<50)
         engine_clock:=1 'dashing sound
       else
        if((carx[tempy1]>>1)+car_size =>36 AND (carx[tempy1]>>1)=<41)
         engine_clock:=1 'dashing sound                                     
   ''check player 2 - x=112
   if(chick1y>16)
      tempy1:=(chick1y-24+8)>>4 'top of chicken
      ''check chicken
      if(tempy1<10)
       case c := car_type[tempy1]
         0: 'small
          car_size:=8
         1: 'small
          car_size:=8
         2: 'medium
          car_size:=16
         3: 'large
          car_size:=32
       if((carx[tempy1]>>1)+car_size =>113 AND (carx[tempy1]>>1)=<117)
           player_fall[1]:=fall_value
           snd.PlaySoundFM(4, snd#SHAPE_NOISE, 200, CONSTANT(snd#SAMPLE_RATE/7),255,$2457_9DEF)
      ''check for sound effect of car coming      
      tempy1:=(chick1y-16-8)>>4 'top of chicken
      ''check top of chicken
      if(engine_clock==0 AND tempy1<10)
       case c := car_type[tempy1]
         0: 'small
          car_size:=8
         1: 'small
          car_size:=8
         2: 'medium
          car_size:=16
         3: 'large
          car_size:=32
'     if((carx[tempy1]>>1)+car_size =>113 AND (carx[tempy1]>>1)=<117)
      if(tempy1<5) ''top half of road
       if((carx[tempy1]>>1)+car_size =>117 AND (carx[tempy1]>>1)=<121)
         engine_clock:=1 'dashing sound
      else
        if((carx[tempy1]>>1)+car_size =>99 AND (carx[tempy1]>>1)=<113)
         engine_clock:=1 'dashing sound                                     
         
      tempy1:=(chick1y-16+8)>>4 'bottom of chicken
      ''check top of chicken
      if(engine_clock==0 AND tempy1<10)
       case c := car_type[tempy1]
         0: 'small
          car_size:=8
         1: 'small
          car_size:=8
         2: 'medium
          car_size:=16
         3: 'large
          car_size:=32
      if(tempy1<5) ''top half of road
       if((carx[tempy1]>>1)+car_size =>117 AND (carx[tempy1]>>1)=<121)
         engine_clock:=1 'dashing sound
      else
        if((carx[tempy1]>>1)+car_size =>99 AND (carx[tempy1]>>1)=<113)
         engine_clock:=1 'dashing sound                                     
                      
PUB Do_Chicken0(X, Y) : anim

    anim := 0

    if (chick0y >> 2 & 1) == 1
      anim := 1

    byte[sptr][SPRX]:=X
    byte[sptr][SPRY]:=Y
    byte[sptr][SPRP]:=1
    byte[sptr][SPRC]:=114 + anim
    sptr+=4

    byte[sptr][SPRX]:=X
    byte[sptr][SPRY]:=Y+8
    byte[sptr][SPRP]:=1
    byte[sptr][SPRC]:=146 + anim
    sptr+=4
PUB Honk_horn(h_number) |  value1, value2

 case h_number
  0:
    value1:=419
    value2:=524
  1:
    value1:=349
    value2:=419  
  2:
    value1:=299
    value2:=349  
  3:
    value1:=262
    value2:=299      
 snd.PlaySoundFM(0, snd#SHAPE_TRIANGLE, value1, CONSTANT(snd#SAMPLE_RATE/2),255,$FFFF_FFFF)
 snd.PlaySoundFM(1, snd#SHAPE_TRIANGLE, value2, CONSTANT(snd#SAMPLE_RATE/2),255,$FFFF_FFFF)
                   
PUB Do_Chicken1(X, Y) : anim

    if (chick1y >> 2 & 1) == 1
      anim := 1

    byte[sptr][SPRX]:=X
    byte[sptr][SPRY]:=Y
    byte[sptr][SPRP]:=1
    byte[sptr][SPRC]:=114 + anim
    sptr+=4

    byte[sptr][SPRX]:=X
    byte[sptr][SPRY]:=Y+8
    byte[sptr][SPRP]:=1
    byte[sptr][SPRC]:=146 + anim
    sptr+=4

PUB Update_Score0 | i
    word[chrmap][5] := word[chrmap][5] + 1
    if (word[chrmap][5] == ("9" + 1))
      word[chrmap][5] := "0"
      if(word[chrmap][4] == $20)
        word[chrmap][4] := "0"       
      word[chrmap][4] := word[chrmap][4] + 1

PUB Update_Score1 | i
    word[chrmap][14] := word[chrmap][14] + 1
    if (word[chrmap][14] == ("9" + 1))
      word[chrmap][14] := "0"
      if(word[chrmap][13] == $20)
        word[chrmap][13] := "0"       
      word[chrmap][13] := word[chrmap][13] + 1

PUB Reset_Score | i
                         
    word[chrmap][14] := 0
    word[chrmap][5] := 0
      word[chrmap][5] := "0"
      word[chrmap][4] := $20
      word[chrmap][14] := "0"
      word[chrmap][13] := $20
    
PUB Do_Copyright_Anim | i,s,d
  if logo_animation==0
     frame:=0
  repeat i from 0 to 7
    s:=@SpriteSet+get_copyright_scan((frame>>1),i)
    d:=@SpriteSet+($f0<<5)+i<<2
    long[d+$00]:=long[s+$00]
    long[d+$20]:=long[s+$20]
    long[d+$40]:=long[s+$40]
    long[d+$60]:=long[s+$60]
    long[d+$80]:=long[s+$80]
    long[d+$a0]:=long[s+$a0]
    
PUB get_copyright_scan(fr,of) | t
  fr&=255
  if(fr<128-8)
    t:=0
  elseif(fr<128)
    t:=fr-120
  elseif(fr<256-8)
    t:=8
  else
    t:=8+(fr-248)
  t:=(t+of)&15
  if(t<8)
    return ($0d<<5)+(t<<2)
  return ($12<<5)+(t<<2)
           
pub VSync
'  bordercolour:=$02
  repeat until VSyncVal<>2
  repeat until VSyncVal==2
'  bordercolour:=$2d
  frame ++
  nframe --
              
DAT

' DO NOT ALTER ANYTHING FROM HERE ESPECIALLY LINE ORDER!
nextline      long      0                        
bordercolour  long      $02
tvparams      long      0               'status
              long      1               'enable
              long      modepins        'pins ' PROTO/DEMO BOARD = %001_0101 ' HYDRA = %011_0000
              long      mode            'mode
              long      10              'hc
              long      12              'vc
              long      myhx            'hx
              long      1               'vx
              long      0               'ho
              long      0               'vo
              long      50_000_000      '_xinfreq<<4  'broadcast
              long      0               'auralcog
scanbufptr    long      0
              long      @bordercolour+$10  'pointer to border colour
              long      @nextline+$10
              long      0
VSyncVal      long      0
dispptr       long      @displist+$10        
sprset        long      @SpriteSet+$10
chrmap        long      @scrmap+$10        
sprpal        long      @SpritePal+$10
cogcount      long      COGS
sprite_count  long      64
sprites       byte      0[64*4] 'this is the sprite list that gets draw
' DO NOT ALTER ANYTHING TO HERE ESPECIALLY LINE ORDER!

'Sprite list is 4 bytes per sprite, bytes are Xpos,Ypos,Palette,Charnum  Xpos 0 = showing at 0 onscreen, Ypos 0 = off top of screen, 8= showing at top line of display, that way when they are 0'd out, they're off screen :D Palette is colour index offset*16, 0=first 16 colours, 1=second 16 colours, Charnum=sprite character index
spritesbuf    byte      0[64*4] 'this is the sprite list buffer, so it can be updated without tears if drawing screen at the same time.

'Displist is made up of 192 * (8 bytes per scanline), each scanline is 5bytes ( 40 bits for each WIDE background pixel ), the next byte is background colour ( 0 in the 40bits ), the next byte is foreground colour ( 1 in the 40 bits ), the last byte is going to be border colour for that scanline.
'if background colour = 0 then it doesn't use the display list pixels, it uses scrmap as an 8x8 character map screen, 20x24 chars in total. each char is indexed from sprset's 16 colour characters
displist      byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$cf,$02 '0
              byte      %00100000,%00000000,%00000000,%00000000,%00000110,$00,$ce,$02
              byte      %00110100,%00000000,%00000000,%00000000,%00101111,$00,$cd,$02
              byte      %00111100,%00000001,%10000000,%00000000,%00111111,$00,$cc,$02
              byte      %00111100,%00000001,%10000000,%00000000,%00111111,$00,$ca,$02 '4
              byte      %00111100,%00000001,%10000000,%00000000,%00111111,$00,$ca,$02
              byte      %00111100,%00000001,%10000000,%00000000,%00111111,$00,$ca,$02
              byte      %00111100,%00000001,%10000000,%00000000,%00111111,$00,$ca,$02                   'Score Text Lines
              
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$02,$ca,$02 '8
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$ca,$02 '12
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$02,$ca,$02 '16
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02 '20
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02 '24
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02 '28
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$06,$02 '32
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02 '36
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02 '40
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ca,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$03,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$03,$02 '44
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$06,$02 '48
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02 '52
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02 '56
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02 '60
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$06,$02 '64
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02 '68
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02 '72
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02 '76
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$06,$02 '80
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cb,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$03,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$03,$02 '84
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '88
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '92
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$7e,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '96       'Middle Of Screen
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$7e,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '100
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '104
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '108
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$06,$02 '112
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '116
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '120
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cc,$02 '124
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$cd,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$ce,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$06,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '128
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '132
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '136
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '140
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$06,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '144
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '148
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '152
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '156
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$04,$06,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '160
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '164
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '168
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02 '172
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$04,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$02,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$00,$02 '176
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$00,$02 '180
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$05,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$02,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$00,$02 '184
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$00,$02 '188
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$00,$02
              byte      %00000000,%00000000,%00000000,%00000000,%00000000,$00,$00,$02

'              byte      %10101010,%10101010,%10101010,%10101010,%10101010,$02,$2b,$02
'              byte      %01010101,%01010101,%01010101,%01010101,%01010101,$02,$2b,$02

scrmap        word      $20,$20,$20,$20,$20,"0",$20,$20,$20,$20,$20,$20,$20,$20,"0",$20,$20,$20,$20,$20 '  0
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 '  8
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 16
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 24
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 32
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 40
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 48
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 56
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 64
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 72
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 80
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 88
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ' 96
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 '104
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 '112
              word      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 '120
              word      $00,$20,$20,$01,$02,$03,$04,$04,$04,$04,$04,$04,$04,$04,$04,$04,$20,$20,$20,$20 '128
              word      $00,$20,$20,$20,$20,$20,$20,$20,$20,$20,$19,$19,$19,$20,$20,$20,$20,$20,$20,$20 '136
              word      $00,$20,$20,$20,$20,$20,$20,$20,$20,$20,$1a,$1a,$1a,$20,$20,$20,$20,$20,$20,$20 '144
              word      $00,$20,$20,$20,$20,$20,$20,$1b,$1b,$1b,$1b,$1b,$1b,$20,$20,$20,$20,$20,$20,$20 '152
              word      $00,$20,$20,$20,$20,$20,$20,$1c,$1c,$1c,$1c,$1c,$1c,$20,$20,$20,$20,$20,$20,$20 '160
              word      $00,$20,$20,$20,$20,$20,$20,"1","0","1","3","2","4",$20,$20,$20,$20,$20,$20,$20 '168
              word      $00,$00,$00,$00,$00,$00,$00,$f0,$f1,$f2,$f3,$f4,$f5,$00,$00,$00,$00,$00,$00,$00 '176
              word      $00,$00,$00,$00,$00,$00,$00,$f0,$f1,$f2,$f3,$f4,$f5,$00,$00,$00,$00,$00,$00,$00 '184

'SpriteSet and SpritePal are generated by the Bmp8ToLite.exe app and an 8bit bmp file called sprset.bmp, using
'Bmp8ToLite sprset -charmap16
'This generates two files, sprset.chr and sprset.pal
SpriteSet     file      "dd1-sprset.chr"
SpritePal     file      "dd1-sprset.pal"
              byte      $02, $03, $04, $05, $06, $07, $75, $86, $89, $38, $45, $02, $03, $04, $05, $06


DAT

game_time        long           0      'Atari 2:20 is the perfect game time.
joysticks        long           0
gstate           long           0      'Game State Flags
chick0y          long           0
chick1y          long           0
player0_score    long           0
player1_score    long           0