/* db_xmsnd.c - xmodem send protocol
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdio.h>
#include "db_xmodem_v010.h"

static int snddata(FILE *fp);

int SendFileToXGS(char *port, char *srcname, char *dstname)
{
    FILE *fp;
	long fsize;
    int sts;

    /* open the file */
    if (!(fp = fopen(srcname, "rb"))) {
		md_done();
		return FALSE;
	}

	/* get the file size */
	fseek(fp, 0, SEEK_END);
    fsize = ftell(fp);

	/* seek back to the start of the file */
	fseek(fp, 0, SEEK_SET);

    /* send the filename header and the data */
	if ((sts = xy_sndhdr(dstname, fsize)) == ER_OK)
        sts = snddata(fp);

    /* close the file */
	fclose(fp);

	/* return successfully */
	return TRUE;
}

/* snddata - send data using x/ymodem */
static int snddata(FILE *fp)
{
    unsigned char buf[SBLKLEN], *bp;
    int bc, ch, sts;

    /* transfer the data */
    if ((sts = xy_sinit()) == ER_OK) {

        /* send each byte */
        for (bp = buf, bc = 0; (ch = getc(fp)) != EOF; ) {
            *bp++ = ch;
            if (++bc == sizeof(buf)) {
                if ((sts = xy_sndblk(buf, sizeof(buf))) != ER_OK)
                    return sts;
                bp = buf;
                bc = 0;
            }
        }

        /* flush partial buffer */
        if (bc > 0) {
            while (++bc <= sizeof(buf))
                *bp++ = '\0';
            if ((sts = xy_sndblk(buf, sizeof(buf))) != ER_OK)
                return sts;
        }

        /* send EOT and wait for ACK */
        sts = xy_sndeot();
    }

    return sts;
}

