/* db_vmfcn.c - intrinsic functions
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdlib.h>
#include "db_vm_v010.h"
#include "db_text_v010.h"

#ifndef WIN32
#include "XGS_PIC_GFX_DRV_V010.h"
#include "XGS_PIC_GAMEPAD_DRV_V010.h"
#include "XGS_PIC_SOUND_DRV_V010.h"
extern unsigned char g_VRAMBuffer[] __attribute__((far));
#endif

static void fcn_rnd(Interpreter *i);
static void fcn_plot(Interpreter *i);
static void fcn_line(Interpreter *i);
static void fcn_hline(Interpreter *i);
static void fcn_vline(Interpreter *i);
static void fcn_clear(Interpreter *i);
static void fcn_fillrect(Interpreter *i);
static void fcn_circle(Interpreter *i);
static void fcn_waitforvsync(Interpreter *i);
static void fcn_gamepad(Interpreter *i);
static void fcn_sound(Interpreter *i);
static void fcn_text(Interpreter *i);
static void fcn_debugout(Interpreter *i);

void InitFcn(Interpreter *i)
{
	i->intrinsics[0] = fcn_rnd;
	i->intrinsics[1] = fcn_plot;
	i->intrinsics[2] = fcn_line;
    i->intrinsics[3] = fcn_hline;
    i->intrinsics[4] = fcn_vline;
	i->intrinsics[5] = fcn_clear;
	i->intrinsics[6] = fcn_fillrect;
	i->intrinsics[7] = fcn_circle;
	i->intrinsics[8] = fcn_waitforvsync;
	i->intrinsics[9] = fcn_gamepad;
	i->intrinsics[10] = fcn_sound;
    i->intrinsics[11] = fcn_text;
	i->intrinsics[12] = fcn_debugout;
}

/* fcn_rnd - RND(n): return a random number between 0 and n-1 */
static void fcn_rnd(Interpreter *i)
{
    int n = i->sp[0];
    i->sp[-1] = rand() % n;
    Drop(i, 1);
}

static void fcn_plot(Interpreter *i)
{
	int x = i->sp[-2];
	int y = i->sp[-1];
	int color = i->sp[0];
#ifdef WIN32
    printf("plot: %d, %d, %d\n",x, y, color);
#else
	GFX_Plot_2BPP(x, y, color, g_VRAMBuffer);
#endif
	Drop(i, 4);
}

static void fcn_line(Interpreter *i)
{
	int x1 = i->sp[-4];
	int y1 = i->sp[-3];
	int x2 = i->sp[-2];
	int y2 = i->sp[-1];
	int color = i->sp[0];
#ifdef WIN32
    printf("line: %d, %d, %d, %d, %d\n", x1, y1, x2, y2, color);
#else
	GFX_Line_2BPP(x1, y1, x2, y2, color, g_VRAMBuffer);
#endif
	Drop(i, 6);
}

static void fcn_hline(Interpreter *i)
{
	int x1 = i->sp[-4];
	int x2 = i->sp[-2];
	int y = i->sp[-1];
	int color = i->sp[0];
#ifdef WIN32
    printf("hline: %d, %d, %d, %d\n", x1, x2, y, color);
#else
	GFX_HLine_2BPP(x1, x2, y, color, g_VRAMBuffer);
#endif
	Drop(i, 5);
}

static void fcn_vline(Interpreter *i)
{
	int x = i->sp[-4];
	int y1 = i->sp[-3];
	int y2 = i->sp[-1];
	int color = i->sp[0];
#ifdef WIN32
    printf("vline: %d, %d, %d, %d\n", x, y1, y2, color);
#else
	GFX_VLine_2BPP(x, y1, y2, color, g_VRAMBuffer);
#endif
	Drop(i, 5);
}

static void fcn_clear(Interpreter *i)
{
	int color = i->sp[0];
#ifdef WIN32
    printf("clear\n");
#else
    GFX_FillScreen_2BPP(color, g_VRAMBuffer);
#endif
	Drop(i, 2);
}

static void fcn_fillrect(Interpreter *i)
{
	int x = i->sp[-4];
	int y = i->sp[-3];
	int w = i->sp[-2];
	int h = i->sp[-1];
	int color = i->sp[0];
#ifdef WIN32
	printf("fillrect: %d, %d, %d, %d, %d\n", x, y, w, h, color);
#else
	GFX_FillRegion_2BPP(x, y, w, h, color, g_VRAMBuffer);
#endif
	Drop(i, 6);
}

static void fcn_circle(Interpreter *i)
{
	int x = i->sp[-3];
	int y = i->sp[-2];
	int r = i->sp[-1];
	int color = i->sp[0];
#ifdef WIN32
	printf("circle: %d, %d, %d, %d\n", x, y, r, color);
#else
	GFX_Circle_2BPP(x, y, r, color, g_VRAMBuffer);
#endif
	Drop(i, 5);
}

static void fcn_waitforvsync(Interpreter *i)
{
#ifdef WIN32
	printf("waitforvsync\n");
#else
	WAIT_FOR_VSYNC_END();	
	WAIT_FOR_VSYNC_START();	
#endif
	Drop(i, 1);
}

static void fcn_gamepad(Interpreter *i)
{
	int which = i->sp[0];
#ifdef WIN32
	printf("gamepad: %d\n", which);
#else
	i->sp[-1] = Gamepad_Read(which == 0 ? GAMEPAD_0 : GAMEPAD_1);
#endif
	Drop(i, 1);
}

static void fcn_sound(Interpreter *i)
{
	int freq = i->sp[0];
#ifdef WIN32
	printf("sound: %d\n", freq);
#else
	SND_PlayTone(freq);
#endif
	Drop(i, 2);
}

static void fcn_text(Interpreter *i)
{
    int x = i->sp[-2];
    int y = i->sp[-1];
	char *str = (char *)i->code + i->sp[0];
#ifdef WIN32
	printf("text (%d,%d): %s\n", x, y, str);
#else
	DrawBitmapFontString(str, x, y, g_VRAMBuffer);
#endif
	Drop(i, 4);
}

static void fcn_debugout(Interpreter *i)
{
	char *str = (char *)i->code + i->sp[0];
	VM_printf("debug: %s\n", str);
	Drop(i, 2);
}
