/* db_vmimage_v010.c - bytecode image file routines
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "db_vm_v010.h"

/* LoadImage - load an image file */
int LoadImage(Interpreter *i, char *name)
{
    FileHdr hdr;
    size_t size;
    VMFILE fp;

    /* open the image file */
    if (!(fp = (*i->fileOpen)(name, "rb")))
{VM_printf("file open failed\n");
        return FALSE;
}

    /* read the image header */
    if ((*i->fileRead)(&hdr, sizeof(hdr), fp) != sizeof(hdr))
{VM_printf("header read failed\n");
        goto fail;
}

    /* verify the image header */
    if (strncmp(hdr.tag, FileTag, strlen(FileTag)) != 0 || hdr.version != FileVersion)
{VM_printf("header verify failed\n");
        goto fail;
}
    /* allocate the code space */
    if (!(i->code = (uint8_t *)malloc(hdr.codeSize)))
{VM_printf("code space allocation failed\n");
        goto fail;
}

    /* allocate the variable space (including the stack) */
    size = (hdr.variableCount + hdr.stackSize) * sizeof(int16_t);
    if (!(i->variables = (int16_t *)malloc(size)))
{VM_printf("variable space allocation failed\n");
        goto fail2;
}

    /* read the code */
    (*i->fileSeek)(fp, hdr.codeOffset, SEEK_SET);
    if ((*i->fileRead)(i->code, hdr.codeSize, fp) != hdr.codeSize)
{VM_printf("code read failed\n");
        goto fail3;
}

    /* read the variable initial values */
    size = hdr.variableCount * sizeof(int16_t);
    (*i->fileSeek)(fp, hdr.variableOffset, SEEK_SET);
    if ((*i->fileRead)(i->variables, size, fp) != size)
{VM_printf("variable read failed\n");
        goto fail3;
}
    
    /* close the image file */
    (*i->fileClose)(fp);

    /* get the offset to the main code */
    i->mainCodeOffset = hdr.mainCodeOffset;
    i->stack = (int16_t *)(i->variables + hdr.variableCount);
    i->stackTop = i->stack + hdr.stackSize;

    /* return successfully */
    return TRUE;

    /* cleanup and report a failure */
fail3:
    free(i->variables);
fail2:
    free(i->code);
fail:
    (*i->fileClose)(fp);
    return FALSE;
}

/* FreeImage - free an image */
void FreeImage(Interpreter *i)
{
    free(i->code);
    free(i->variables);
}

