/* db_bcode.c - bytecode handling functions
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include "db_bascom_v010.h"

/* codeaddr - get the current code address (actually, offset) */
int codeaddr(ParseContext *c)
{
    return (int)(c->cptr - c->codeBuf);
}

/* putcbyte - put a code byte into the code buffer */
int putcbyte(ParseContext *c, int b)
{
    int addr = codeaddr(c);
    if (c->cptr >= c->ctop)
        Fatal("Bytecode buffer overflow");
    *c->cptr++ = b;
    return addr;
}

/* putcword - put a code word into the code buffer */
int putcword(ParseContext *c, int w)
{
    int addr = codeaddr(c);
    if (c->cptr + 2 > c->ctop)
        Fatal("Bytecode buffer overflow");
    *c->cptr++ = w;
    *c->cptr++ = w >> 8;
    return addr;
}

/* fixup - fixup a reference chain */
void fixup(ParseContext *c, int chn, int val)
{
    while (chn != 0) {
        int nxt = (int16_t)(c->codeBuf[chn] | (c->codeBuf[chn + 1] << 8));
        c->codeBuf[chn] = val;
        c->codeBuf[chn + 1] = val >> 8;
        chn = nxt;
    }
}

/* fixupbranch - fixup a reference chain */
void fixupbranch(ParseContext *c, int chn, int val)
{
    while (chn != 0) {
        int nxt = (int16_t)(c->codeBuf[chn] | (c->codeBuf[chn + 1] << 8));
        int off = val - (chn + 2); /* this assumes all three byte branch instructions */
        c->codeBuf[chn] = off;
        c->codeBuf[chn + 1] = off >> 8;
        chn = nxt;
    }
}
