#include <stdio.h>
#include "../runtime/db_vm.h"

#define VMCODE_SIZE		4096
static uint8_t vmcode[VMCODE_SIZE];

#define VMDATA_SIZE		4096
static uint8_t vmdata[VMDATA_SIZE];	

Interpreter interp;

int main(int argc, char *argv[])
{
    Interpreter *i = &interp;

    InitInterpreter(i, TARGET_TILE);
    InitFcn(i);

    if (LoadImage(i, argv[1]))
        Execute(i);

    return 0;
}

/* VM_LoadMemoryImageHeader - load the header of an image already in memory */
int VM_LoadMemoryImageHeader(Interpreter *i)
{
    return VMFALSE;
}

/* VM_StoreMemoryImageHeader - store the header of an image in memory */
void VM_StoreMemoryImageHeader(Interpreter *i)
{
}

/* VM_GetMemoryImageInitData - load initialization data of an image already in memory */
uint8_t *VM_GetMemoryImageInitData(Interpreter *i)
{
    return NULL;
}

/* VM_AllocateImage - allocate space for an image */
int VM_AllocateImage(Interpreter *i, int codeSize, int initSize, int dataSize)
{
    /* make sure there is enough memory */
    if (codeSize > VMCODE_SIZE || dataSize > VMDATA_SIZE)
        return VMFALSE;

    /* setup the code and data space */
    i->code = vmcode;
    i->variables = (int16_t *)vmdata;

	/* return successfully */
	return VMTRUE;
}

/* VM_FreeImage - free an image */
void VM_FreeImage(Interpreter *i)
{
}

/* VM_LoadCode - load code into memory */
void VM_LoadCode(uint16_t dstoff, uint8_t *src, int cnt)
{
    memcpy(interp.code + dstoff, src, cnt);
}

/* VM_LoadInit - load initialization data into flash */
void VM_LoadInit(uint16_t dstoff, uint8_t *src, int cnt)
{
    /* don't need to save initialization data because we can't restart */
}

/* VM_ReadInit - read initialization data from flash */
void VM_ReadInit(uint16_t off, uint8_t *buf, int cnt)
{
    /* no saved initialization data to load */
}

void VM_printf(char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    vprintf(fmt, ap);
    va_end(ap);
}

void VM_vprintf(char *fmt, va_list ap)
{
	vprintf(fmt, ap);
}

void VM_putchar(int ch)
{
	putchar(ch);
}

void VM_flush(void)
{
	fflush(stdout);
}

VMFILE *VM_fopen(const char *name, const char *mode)
{
    return (VMFILE *)fopen(name, mode);
}

int VM_fclose(VMFILE *file)
{
    return fclose((FILE *)file);
}

size_t VM_fread(void *buf, size_t count, VMFILE *file)
{
    return fread(buf, 1, count, (FILE *)file);
}

size_t VM_fwrite(void *buf, size_t count, VMFILE *file)
{
    return fwrite(buf, 1, count, (FILE *)file);
}

int VM_fseek(VMFILE *file, long offset, int whence)
{
    return fseek((FILE *)file, offset, whence);
}

long VM_ftell(VMFILE *file)
{
    return ftell((FILE *)file);
}

void VM_poll(void)
{
}
