/* db_mdio.c - serial i/o routines
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include "windows.h"

#include <stdio.h>
#include <stdarg.h>
#include "../common/db_mdio.h"

static HANDLE hSerial;
static COMMTIMEOUTS timeouts;

int md_init(const char *port, unsigned long baud)
{
    char fullPort[20];
    DCB state;

    sprintf(fullPort, "\\\\.\\%s", port);

    hSerial = CreateFile(
        fullPort,
        GENERIC_READ | GENERIC_WRITE,
        0,
        NULL,
        OPEN_EXISTING,
        0,
        NULL);

    if (hSerial == INVALID_HANDLE_VALUE)
        return VMFALSE;

    GetCommState(hSerial, &state);
    switch (baud) {
    case 9600:
        state.BaudRate = CBR_9600;
        break;
    case 19200:
        state.BaudRate = CBR_19200;
        break;
    case 38400:
        state.BaudRate = CBR_38400;
        break;
    case 57600:
        state.BaudRate = CBR_57600;
        break;
    case 115200:
        state.BaudRate = CBR_115200;
        break;
    default:
        return VMFALSE;
    }
    state.ByteSize = 8;
    state.Parity = NOPARITY;
    state.StopBits = ONESTOPBIT;
    state.fOutxDsrFlow = VMFALSE;
    state.fDtrControl = DTR_CONTROL_ENABLE;
    state.fOutxCtsFlow = VMFALSE;
    state.fRtsControl = RTS_CONTROL_DISABLE;
    state.fInX = VMFALSE;
    state.fOutX = VMFALSE;
    state.fBinary = VMTRUE;
    state.fParity = VMFALSE;
    state.fDsrSensitivity = VMFALSE;
    state.fTXContinueOnXoff = VMTRUE;
    state.fNull = VMFALSE;
    state.fAbortOnError = VMFALSE;
    SetCommState(hSerial, &state);

    GetCommTimeouts(hSerial, &timeouts);
    timeouts.ReadIntervalTimeout = MAXDWORD;
    timeouts.ReadTotalTimeoutMultiplier = MAXDWORD;

	/* setup device buffers */
	SetupComm(hSerial, 10000, 10000);

	/* purge any information in the buffer */
	PurgeComm(hSerial, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);

    return VMTRUE;
}

void md_done(void)
{
	FlushFileBuffers(hSerial);
    CloseHandle(hSerial);
}

int md_write(uint8_t *buf, int len)
{
    DWORD written;
    BOOL ok;
	ok = WriteFile(hSerial, (void *)buf, (DWORD)len, &written, NULL);
    return ok && (int)written == len;
}

int md_get(int timeout)
{
    unsigned char buf[1];
    DWORD read;
    BOOL ok;
    timeouts.ReadTotalTimeoutConstant = timeout * 1000;
    SetCommTimeouts(hSerial, &timeouts);
    ok = ReadFile(hSerial, (void *)buf, (DWORD)1, &read, NULL);
    return ok && read == 1 ? buf[0] : MD_TIMEOUT;
}

void md_reset(void)
{
    EscapeCommFunction(hSerial, SETDTR);
    Sleep(100);
    EscapeCommFunction(hSerial, CLRDTR);
    Sleep(100);
}
