#include <windows.h>
#include "psapi.h"
#include "db_compiler.h"

static char *GetFileExtension(const char *file);

/* ConstructOutputName - construct an output filename from an input filename */
void ConstructOutputName(const char *infile, char *outfile)
{
    char *end = strrchr(infile, '.');
    if (end && !strchr(end, '/') && !strchr(end, '\\')) {
        strncpy(outfile, infile, end - infile);
        outfile[end - infile] = '\0';
    }
    else
        strcpy(outfile, infile);
    strcat(outfile, ".bai");
}

/* IsSourceFile - check for a source file name */
int IsSourceFile(const char *file)
{
    return strcasecmp(GetFileExtension(file), ".bas") == 0;
}

/* IsImageFile - check for an image file name */
int IsImageFile(const char *file)
{
    return strcasecmp(GetFileExtension(file), ".bai") == 0;
}

/* GetFileExtension - get the file extension */
static char *GetFileExtension(const char *file)
{
    char *end = strrchr(file, '.');
    return end ? end : "";
}

/* GetIncludePath - get the full path to a file in the include directory */
const char *GetIncludePath(const char *name)
{
    static char path[1024];
    char *p;
    
    /* get the full path to the executable */
    if (!GetModuleFileNameEx(GetCurrentProcess(), NULL, path, sizeof(path)))
        return name;

    /* remove the executable filename */
    if ((p = strrchr(path, '\\')) != NULL)
        *p = '\0';

    /* remove the immediate directory containing the executable (usually 'bin') */
    if ((p = strrchr(path, '\\')) != NULL) {
        *p = '\0';
        
        /* check for the 'Release' or 'Debug' build directories used by Visual C++ */
        if (strcmp(&p[1], "Release") == 0 || strcmp(&p[1], "Debug") == 0) {
            if ((p = strrchr(path, '\\')) != NULL)
                *p = '\0';
        }
    }

    /* generate the full path to the system include file */
    strcat(path, "\\include\\");
    strcat(path, name);
    return path;
}
