#include <stdio.h>

typedef unsigned char uint8_t;
typedef unsigned short uint16_t;

#define TYPE_DATA   0x00
#define TYPE_EOF    0x01
#define TYPE_EADDR  0x04

#define LOAD_ADDR   0x00010000

uint8_t rec_length;
uint16_t rec_addr;
uint8_t rec_type;
uint8_t rec_data[16];
uint8_t rec_checksum;

static void InitRecord(uint8_t type, uint16_t addr);
static void WriteByte(FILE *fp, uint8_t byte);
static void FlushRecord(FILE *fp);

int main(int argc, char *argv[])
{
    FILE *ifp, *ofp;
    int byte, cnt = 0;

	if (argc != 3) {
		fprintf(stderr, "usage: bai2hex infile outfile\n");
		exit(1);
	}
	if (!(ifp = fopen(argv[1], "rb"))) {
		fprintf(stderr, "error: can't open %s\n", argv[1]);
		exit(1);
	}
	if (!(ofp = fopen(argv[2], "w"))) {
		fprintf(stderr, "error: can't create %s\n", argv[2]);
		exit(1);
	}
	
    InitRecord(TYPE_EADDR, 0x0000);
    WriteByte(ofp, (LOAD_ADDR >> 24) & 0xff);
    WriteByte(ofp, (LOAD_ADDR >> 16) & 0xff);
    FlushRecord(ofp);

    InitRecord(TYPE_DATA, LOAD_ADDR & 0x0000ffff);
    while ((byte = getc(ifp)) != EOF) {
        WriteByte(ofp, (uint8_t)byte);
        if ((++cnt & 1) == 0) {
            WriteByte(ofp, 0x00);
            WriteByte(ofp, 0x00);
        }
    }
    FlushRecord(ofp);
    
    InitRecord(TYPE_EOF, 0x0000);
    FlushRecord(ofp);
    
    fclose(ifp);
    fclose(ofp);

    return 0;
}

static void InitRecord(uint8_t type, uint16_t addr)
{
    rec_length = 0;
    rec_addr = addr;
    rec_type = type;
    rec_checksum = 0;
}

static void WriteByte(FILE *fp, uint8_t byte)
{
    rec_data[rec_length++] = byte;
    rec_checksum += byte;
    if (rec_length == sizeof(rec_data))
        FlushRecord(fp);
}

static void FlushRecord(FILE *fp)
{
    int i;
    rec_checksum += rec_length;
    rec_checksum += rec_addr >> 8;
    rec_checksum += rec_addr & 0xff;
    rec_checksum += rec_type;
    fprintf(fp,":%02x%04x%02x", rec_length, rec_addr, rec_type);
    for (i = 0; i < rec_length; ++i)
        fprintf(fp, "%02x", rec_data[i]);
    fprintf(fp,"%02x\n", (~rec_checksum + 1) & 0xff);
    rec_addr += rec_length;
    rec_length = 0;
    rec_checksum = 0;
}
