#include <stdio.h>
#include <math.h>

#define M_MIN		2.0
#define M_MAX		513.0
#define N1_MIN	2.0
#define N1_MAX	33.0
#define N2_MIN	2.0
#define N2_MAX	8.0

int main(int argc, char *argv[])
{
	double Fin, Fosc, Fcy, target_Fcy, M, N1, N2;
	double best_M, best_N1, best_N2, best_Fcy;
	
	if (argc != 3) {
		fprintf(stderr, "usage: pll <osc-freq> <clock-freq>\n");
		exit(1);
	}
	
	Fin = atof(argv[1]);
	target_Fcy = atof(argv[2]);
	
	best_Fcy = -1.0;
	
	for (M = M_MIN; M <= M_MAX; ++M) {
		for (N1 = N1_MIN; N1 <= N1_MAX; ++N1) {
			for (N2 = N2_MIN; N2 <= N2_MAX; N2 *= N2) {
				double T1, T2;
				// apply the prescaler
				T1 = Fin / N1;
				// apply the feedback divisor
				T2 = T1 * M;
				if (T2 < 100000000.0 || T2 > 200000000.0)
					continue;
				// apply the postscaler
				Fosc = T2 / N2;
				// determine the clock frequency
				Fcy = Fosc / 2.0;
				// check to see if this is better than the best so far
				if (fabs(target_Fcy - Fcy) < fabs(target_Fcy - best_Fcy)) {
					best_Fcy = Fcy;
					best_M = M;
					best_N1 = N1;
					best_N2 = N2;
				}
			}
		}
	}
	
	printf("Fcy: %g\n", best_Fcy);
	printf("M:   %d\n", (int)best_M);
	printf("N1:  %d\n", (int)best_N1);
	printf("N2:  %d\n", (int)best_N2);
	
	return 0;
}
