/* vmlibchamavr.c - a virtual machine library for the Chameleon AVR
 *
 * Copyright (c) 2009-2010 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdio.h>
#include "vmlib.h"
#include "avruart.h"			

void VM_libinit(unsigned long baudRate, VMCODEBASE_t imageBase)
{
    UART_init(baudRate);
}

int VM_getchar(void)
{
    int ch;
    while ((ch = UART_getc()) == -1)
        ;
    return ch;
}

void VM_putchar(int ch)
{
    if (ch == '\n')
        UART_putc('\r');
    UART_putc(ch);
}

void vmcodecopy(uint8_t *dst, const uint8_t *src, int cnt)
{
    while (--cnt >= 0)
        *dst++ = VMCODEBYTE(src++);
}

int vmstrlen(const uint8_t *str)
{
    int len = 0;
    while (VMCODEBYTE(str++) != '\0')
        ++len;
    return len;
}

char *itoa(int value, char *string, int radix)
{
    switch (radix) {
    case 8:
        sprintf(string, "%o", value);
        break;
    case 10:
        sprintf(string, "%d", value);
        break;
    case 16:
        sprintf(string, "%x", value);
        break;
    default:
        strcpy(string, "<unknown radix>");
        break;
    }
    return string;
}
