/* vmlibchampic.c - a virtual machine library for the Chameleon PIC24/dsPIC33
 *
 * Copyright (c) 2009-2010 by David Michael Betz.  All rights reserved.
 *
 */

#include "CHAM_PIC_SYSTEM_V010.h"
#include "CHAM_PIC_UART_DRV_V010.h"
#include <stdio.h>
#include "vmlib.h"

void VM_libinit(unsigned long baudRate, VMCODEBASE_t imageBase)
{
    SYS_ConfigureClock(MAX_FCY_RATE);
    UART_Init(baudRate);
    TBLPAG = imageBase;
}

int VM_getchar(void)
{
    unsigned char ch;
    while (!UART_getchar(&ch))
        ;
    return ch;
}

void VM_putchar(int ch)
{
    if (ch == '\n')
        UART_putchar('\r');
    UART_putchar(ch);
}

void vmcodecopy(uint8_t *dst, const uint8_t *src, int cnt)
{
    while (--cnt >= 0)
        *dst++ = VMCODEBYTE(src++);
}

int vmstrlen(const uint8_t *str)
{
    int len = 0;
    while (VMCODEBYTE(str++) != '\0')
        ++len;
    return len;
}

char *itoa(int value, char *string, int radix)
{
    switch (radix) {
    case 8:
        sprintf(string, "%o", value);
        break;
    case 10:
        sprintf(string, "%d", value);
        break;
    case 16:
        sprintf(string, "%x", value);
        break;
    default:
        strcpy(string, "<unknown radix>");
        break;
    }
    return string;
}
