///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: XGS_AVR_GFX_LIB_V010.c
// 
// Last Modified: 9.1.08
//
// Description: 
//
// Overview:
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// define chip type here
#define __AVR_ATmega644__

// define CPU frequency in Mhz here if not defined in Makefile, this is used for conditional compilation, so needs
// to be first, before other XGS includes, also note there is a copy of this constant in XGS_AVR_SYSTEM_V010, so if you 
// change it here, also change it in the header to make certain everthing is sync'ed
#define F_CPU 28636360UL

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <avr/eeprom.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <compat/twi.h>
#include <avr/pgmspace.h>

// include local XGS API header files
#include "XGS_AVR_SYSTEM_V010.h"
#include "XGS_AVR_GFX_LIB_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES AND MACROS /////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES //////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// storage for our lookup tables, for now declare as size 1 until FLASH is working
float cos_look[1]; // 361, 1 extra so we can store 0-360 inclusive
float sin_look[1]; // 361, 1 extra so we can store 0-360 inclusive

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

volatile extern UCHAR *tile_map_ptr;
volatile extern UCHAR *tile_bitmaps_ptr;
volatile extern UCHAR *vram_ptr;

// screen screen variable for bitmap and tile modes
extern int g_screen_width;
extern int g_screen_height;

extern int g_tile_map_width;
int g_tile_map_height;

// clipping variables
extern int g_clip_min_x;
extern int g_clip_max_x;
extern int g_clip_min_y;
extern int g_clip_max_y;

// these globals keep track of the tile map console printing cursor position
extern int g_tmap_cursor_x;
extern int g_tmap_cursor_y;

// all of these are exported from the ASM drivers always
volatile extern unsigned int  curr_line;
volatile extern unsigned char curr_raster_line;
volatile extern unsigned char tile_map_logical_width;
volatile extern char          tile_map_vscroll;
volatile extern unsigned char palette_table[];

// single missile for VGA tile mode 1 (exported from ASM driver)
volatile extern unsigned char missile_x;
volatile extern unsigned char missile_y; 
volatile extern unsigned char missile_color;
volatile extern unsigned char missile_state;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS  /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GRAPHICS FUNCTIONS /////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GFX_Draw_Top_Tri(int x1,int y1,
                      int x2,int y2,
                      int x3,int y3,
                      int color,
                      UCHAR *dest_buffer)
{
// this function draws a triangle that has a flat top

float dx_right,    // the dx/dy ratio of the right edge of line
      dx_left,     // the dx/dy ratio of the left edge of line
      xs,xe,       // the starting and ending points of the edges
      height;      // the height of the triangle

int temp_x,        // used during sorting as temps
    temp_y,
    right,         // used by clipping
    left;
    
// destination address of next scanline
//UCHAR  *dest_addr = NULL;

// test order of x1 and x2
if (x2 < x1)
   {
   temp_x = x2;
   x2     = x1;
   x1     = temp_x;
   } // end if swap

// compute delta's
height = y3-y1;

dx_left  = (x3-x1)/height;
dx_right = (x3-x2)/height;

// set starting points
xs = (float)x1;
xe = (float)x2+(float)0.5;

// perform y clipping
if (y1 < g_clip_min_y)
   {
   // compute new xs and ys
   xs = xs+dx_left*(float)(-y1+g_clip_min_y);
   xe = xe+dx_right*(float)(-y1+g_clip_min_y);

   // reset y1
   y1=g_clip_min_y;

   } // end if top is off screen

if (y3>g_clip_max_y)
   y3=g_clip_max_y;

// compute starting address in video memory
//dest_addr = dest_buffer+y1*mempitch;

// test if x clipping is needed
if (x1>=g_clip_min_x && x1<=g_clip_max_x &&
    x2>=g_clip_min_x && x2<=g_clip_max_x &&
    x3>=g_clip_min_x && x3<=g_clip_max_x)
    {
    // draw the triangle
    for (temp_y=y1; temp_y<=y3; temp_y++) //, dest_addr+=mempitch)
        {
        // draw segment
        // memset((UCHAR *)dest_addr+(unsigned int)xs, color,(unsigned int)((int)xe-(int)xs+1));
        GFX_Draw_HLine( (int)xs, (int)xe, temp_y, color, dest_buffer);
               
        // adjust starting point and ending point
        xs+=dx_left;
        xe+=dx_right;

        } // end for

    } // end if no x clipping needed
else
   {
   // clip x axis with slower version

   // draw the triangle
   for (temp_y=y1; temp_y<=y3; temp_y++) // ,dest_addr+=mempitch)
       {
       // do x clip
       left  = (int)xs;
       right = (int)xe;

       // adjust starting point and ending point
       xs+=dx_left;
       xe+=dx_right;

       // clip line
       if (left < g_clip_min_x)
          {
          left = g_clip_min_x;

          if (right < g_clip_min_x)
             continue;
          }

       if (right > g_clip_max_x)
          {
          right = g_clip_max_x;

          if (left > g_clip_max_x)
             continue;
          }
       // draw segment
       //memset((UCHAR  *)dest_addr+(unsigned int)left, color,(unsigned int)(right-left+1));
       GFX_Draw_HLine( (int)left, (int)right, temp_y, color, dest_buffer);

       } // end for

   } // end else x clipping needed

} // end GFX_Draw_Top_Tri

/////////////////////////////////////////////////////////////////////////

void GFX_Draw_Bottom_Tri(int x1,int y1,
                         int x2,int y2,
                         int x3,int y3,
                         int color,
                         UCHAR *dest_buffer)
{
// this function draws a triangle that has a flat bottom

float dx_right,    // the dx/dy ratio of the right edge of line
      dx_left,     // the dx/dy ratio of the left edge of line
      xs,xe,       // the starting and ending points of the edges
      height;      // the height of the triangle

int temp_x,        // used during sorting as temps
    temp_y,
    right,         // used by clipping
    left;

// destination address of next scanline
//UCHAR  *dest_addr;

// test order of x1 and x2
if (x3 < x2)
   {
   temp_x = x2;
   x2     = x3;
   x3     = temp_x;
   } // end if swap

// compute delta's
height = y3-y1;

dx_left  = (x2-x1)/height;
dx_right = (x3-x1)/height;

// set starting points
xs = (float)x1;
xe = (float)x1; // +(float)0.5;

// perform y clipping
if (y1<g_clip_min_y)
   {
   // compute new xs and ys
   xs = xs+dx_left*(float)(-y1+g_clip_min_y);
   xe = xe+dx_right*(float)(-y1+g_clip_min_y);

   // reset y1
   y1=g_clip_min_y;

   } // end if top is off screen

if (y3>g_clip_max_y)
   y3=g_clip_max_y;

// compute starting address in video memory
//dest_addr = dest_buffer+y1*mempitch;

// test if x clipping is needed
if (x1>=g_clip_min_x && x1<=g_clip_max_x &&
    x2>=g_clip_min_x && x2<=g_clip_max_x &&
    x3>=g_clip_min_x && x3<=g_clip_max_x)
    {
    // draw the triangle
    for (temp_y=y1; temp_y<=y3; temp_y++) //,dest_addr+=mempitch)
        {
        // draw segment
        //memset((UCHAR  *)dest_addr+(unsigned int)xs, color,(unsigned int)((int)xe-(int)xs+1));
        GFX_Draw_HLine( (int)xs, (int)xe, temp_y, color, dest_buffer);

        // adjust starting point and ending point
        xs+=dx_left;
        xe+=dx_right;

        } // end for

    } // end if no x clipping needed
else
   {
   // clip x axis with slower version

   // draw the triangle

   for (temp_y=y1; temp_y<=y3; temp_y++) // ,dest_addr+=mempitch)
       {
       // do x clip
       left  = (int)xs;
       right = (int)xe;

       // adjust starting point and ending point
       xs+=dx_left;
       xe+=dx_right;

       // clip line
       if (left < g_clip_min_x)
          {
          left = g_clip_min_x;

          if (right < g_clip_min_x)
             continue;
          }

       if (right > g_clip_max_x)
          {
          right = g_clip_max_x;

          if (left > g_clip_max_x)
             continue;
          }

       // draw segment
       //memset((UCHAR  *)dest_addr+(unsigned int)left, color,(unsigned int)(right-left+1));
       GFX_Draw_HLine( (int)left, (int)right, temp_y, color, dest_buffer);

       } // end for

   } // end else x clipping needed

} // end GFX_Draw_Bottom_Tri

/////////////////////////////////////////////////////////////////////////

void GFX_Draw_Triangle_2D(int x1,int y1,
                          int x2,int y2,
                          int x3,int y3,
                          int color,
					      UCHAR *dest_buffer)
{
// this function draws a triangle on the destination buffer
// it decomposes all triangles into a pair of flat top, flat bottom

int temp_x, // used for sorting
    temp_y,
    new_x;


// test for h lines and v lines
if ((x1==x2 && x2==x3)  ||  (y1==y2 && y2==y3))
   return;

// sort p1,p2,p3 in ascending y order
if (y2<y1)
   {
   temp_x = x2;
   temp_y = y2;
   x2     = x1;
   y2     = y1;
   x1     = temp_x;
   y1     = temp_y;
   } // end if

// now we know that p1 and p2 are in order
if (y3<y1)
   {
   temp_x = x3;
   temp_y = y3;
   x3     = x1;
   y3     = y1;
   x1     = temp_x;
   y1     = temp_y;
   } // end if

// finally test y3 against y2
if (y3<y2)
   {
   temp_x = x3;
   temp_y = y3;
   x3     = x2;
   y3     = y2;
   x2     = temp_x;
   y2     = temp_y;

   } // end if

// do trivial rejection tests for clipping
if ( y3<g_clip_min_y || y1>g_clip_max_y ||
    (x1<g_clip_min_x && x2<g_clip_min_x && x3<g_clip_min_x) ||
    (x1>g_clip_max_x && x2>g_clip_max_x && x3>g_clip_max_x) )
   return;

// test if top of triangle is flat
if (y1==y2)
   {
   GFX_Draw_Top_Tri(x1,y1,x2,y2,x3,y3,color, dest_buffer);
   } // end if
else
if (y2==y3)
   {
   GFX_Draw_Bottom_Tri(x1,y1,x2,y2,x3,y3,color, dest_buffer);
   } // end if bottom is flat
else
   {
   // general triangle that's needs to be broken up along long edge
   new_x = x1 + (int)(0.5+(float)(y2-y1)*(float)(x3-x1)/(float)(y3-y1));

   // draw each sub-triangle
   GFX_Draw_Bottom_Tri(x1,y1,new_x,y2,x2,y2,color, dest_buffer);
   GFX_Draw_Top_Tri(x2,y2,new_x,y2,x3,y3,color, dest_buffer);

   } // end else

} // end GFX_Draw_Triangle_2D

/////////////////////////////////////////////////////////////////////////

void Draw_TriangleFP_2D(int x1,int y1,
                        int x2,int y2,
                        int x3,int y3,
                        int color,
    	   			    UCHAR *dest_buffer, int mempitch)
{
// this function draws a triangle on the destination buffer using fixed point
// it decomposes all triangles into a pair of flat top, flat bottom

int temp_x, // used for sorting
    temp_y,
    new_x;

#ifdef DEBUG_ON
	// track rendering stats
    debug_polys_rendered_per_frame++;
#endif


// test for h lines and v lines
if ((x1==x2 && x2==x3)  ||  (y1==y2 && y2==y3))
   return;

// sort p1,p2,p3 in ascending y order
if (y2<y1)
   {
   temp_x = x2;
   temp_y = y2;
   x2     = x1;
   y2     = y1;
   x1     = temp_x;
   y1     = temp_y;
   } // end if

// now we know that p1 and p2 are in order
if (y3<y1)
   {
   temp_x = x3;
   temp_y = y3;
   x3     = x1;
   y3     = y1;
   x1     = temp_x;
   y1     = temp_y;
   } // end if

// finally test y3 against y2
if (y3<y2)
   {
   temp_x = x3;
   temp_y = y3;
   x3     = x2;
   y3     = y2;
   x2     = temp_x;
   y2     = temp_y;

   } // end if

// do trivial rejection tests for clipping
if ( y3<g_clip_min_y || y1>g_clip_max_y ||
    (x1<g_clip_min_x && x2<g_clip_min_x && x3<g_clip_min_x) ||
    (x1>g_clip_max_x && x2>g_clip_max_x && x3>g_clip_max_x) )
   return;

// test if top of triangle is flat
if (y1==y2)
   {
   GFX_Draw_Top_TriFP(x1,y1,x2,y2,x3,y3,color, dest_buffer);
   } // end if
else
if (y2==y3)
   {
   GFX_Draw_Bottom_TriFP(x1,y1,x2,y2,x3,y3,color, dest_buffer);
   } // end if bottom is flat
else
   {
   // general triangle that's needs to be broken up along long edge
   new_x = x1 + (int)(0.5+(float)(y2-y1)*(float)(x3-x1)/(float)(y3-y1));

   // draw each sub-triangle
   GFX_Draw_Bottom_TriFP(x1,y1,new_x,y2,x2,y2,color, dest_buffer);
   GFX_Draw_Top_TriFP(x2,y2,new_x,y2,x3,y3,color, dest_buffer);

   } // end else

} // end Draw_TriangleFP_2D

/////////////////////////////////////////////////////////////////////////

void GFX_Draw_Top_TriFP(int x1,int y1,
                        int x2,int y2,
                        int x3,int y3,
                        int color,
                        UCHAR *dest_buffer)
{
// this function draws a triangle that has a flat top using fixed point math

long dx_right,    // the dx/dy ratio of the right edge of line
     dx_left,     // the dx/dy ratio of the left edge of line
     xs,xe,       // the starting and ending points of the edges
     height;      // the height of the triangle

long temp_x,        // used during sorting as temps
     temp_y,
     right,         // used by clipping
     left;

//UCHAR  *dest_addr;

// test for degenerate
if (y1==y3 || y2==y3)
	return;

// test order of x1 and x2
if (x2 < x1)
   {
   temp_x = x2;
   x2     = x1;
   x1     = temp_x;
   } // end if swap

// compute delta's
height = y3-y1;

dx_left  = ((x3-x1)<<FIXP16_SHIFT)/height;
dx_right = ((x3-x2)<<FIXP16_SHIFT)/height;

// set starting points
xs = (x1<<FIXP16_SHIFT);
xe = (x2<<FIXP16_SHIFT);

// perform y clipping
if (y1<g_clip_min_y)
   {
   // compute new xs and ys
   xs = xs+dx_left*(-y1+g_clip_min_y);
   xe = xe+dx_right*(-y1+g_clip_min_y);

   // reset y1
   y1=g_clip_min_y;

   } // end if top is off screen

if (y3>g_clip_max_y)
   y3=g_clip_max_y;

// compute starting address in video memory
//dest_addr = dest_buffer+y1*mempitch;

// test if x clipping is needed
if (x1>=g_clip_min_x && x1<=g_clip_max_x &&
    x2>=g_clip_min_x && x2<=g_clip_max_x &&
    x3>=g_clip_min_x && x3<=g_clip_max_x)
    {
    // draw the triangle
    for (temp_y=y1; temp_y<=y3; temp_y++ )//,dest_addr+=mempitch)
        {
      // memset((UCHAR *)dest_addr+((xs+FIXP16_ROUND_UP)>>FIXP16_SHIFT),  color, (((xe-xs+FIXP16_ROUND_UP)>>FIXP16_SHIFT)+1));

        // adjust starting point and ending point
        xs+=dx_left;
        xe+=dx_right;

        } // end for

    } // end if no x clipping needed
else
   {
   // clip x axis with slower version

   // draw the triangle
   for (temp_y=y1; temp_y<=y3; temp_y++) //,dest_addr+=mempitch)
       {
       // do x clip
       left  = ((xs+FIXP16_ROUND_UP)>>16);
       right = ((xe+FIXP16_ROUND_UP)>>16);

       // adjust starting point and ending point
       xs+=dx_left;
       xe+=dx_right;

       // clip line
       if (left < g_clip_min_x)
          {
          left = g_clip_min_x;

          if (right < g_clip_min_x)
             continue;
          }

       if (right > g_clip_max_x)
          {
          right = g_clip_max_x;

          if (left > g_clip_max_x)
             continue;
          }

       //memset((UCHAR  *)dest_addr+(unsigned int)left, color,(unsigned int)(right-left+1));

       } // end for

   } // end else x clipping needed

} // end GFX_Draw_Top_TriFP

/////////////////////////////////////////////////////////////////////////

void GFX_Draw_Bottom_TriFP(int x1,int y1,
                           int x2,int y2,
                           int x3,int y3,
                           int color,
                           UCHAR *dest_buffer)
{

// this function draws a triangle that has a flat bottom using fixed point math

long dx_right,    // the dx/dy ratio of the right edge of line
     dx_left,     // the dx/dy ratio of the left edge of line
     xs,xe,       // the starting and ending points of the edges
     height;      // the height of the triangle

long temp_x,        // used during sorting as temps
     temp_y,
     right,         // used by clipping
     left;

//UCHAR  *dest_addr;

if (y1==y2 || y1==y3)
	return;

// test order of x1 and x2
if (x3 < x2)
   {
   temp_x = x2;
   x2     = x3;
   x3     = temp_x;

   } // end if swap

// compute delta's
height = y3-y1;

dx_left  = ((x2-x1)<<FIXP16_SHIFT)/height;
dx_right = ((x3-x1)<<FIXP16_SHIFT)/height;

// set starting points
xs = (x1<<FIXP16_SHIFT);
xe = (x1<<FIXP16_SHIFT);

// perform y clipping
if (y1<g_clip_min_y)
   {
   // compute new xs and ys
   xs = xs+dx_left*(-y1+g_clip_min_y);
   xe = xe+dx_right*(-y1+g_clip_min_y);

   // reset y1
   y1=g_clip_min_y;

   } // end if top is off screen

if (y3>g_clip_max_y)
   y3=g_clip_max_y;

// compute starting address in video memory
//dest_addr = dest_buffer+y1*mempitch;

// test if x clipping is needed
if (x1>=g_clip_min_x && x1<=g_clip_max_x &&
    x2>=g_clip_min_x && x2<=g_clip_max_x &&
    x3>=g_clip_min_x && x3<=g_clip_max_x)
    {
    // draw the triangle
    for (temp_y=y1; temp_y<=y3; temp_y++)//,dest_addr+=mempitch)
        {
        //memset((UCHAR *)dest_addr+((xs+FIXP16_ROUND_UP)>>FIXP16_SHIFT), color, (((xe-xs+FIXP16_ROUND_UP)>>FIXP16_SHIFT)+1));

        // adjust starting point and ending point
        xs+=dx_left;
        xe+=dx_right;

        } // end for

    } // end if no x clipping needed
else
   {
   // clip x axis with slower version

   // draw the triangle
   for (temp_y=y1; temp_y<=y3; temp_y++)//,dest_addr+=mempitch)
       {
       // do x clip
       left  = ((xs+FIXP16_ROUND_UP)>>FIXP16_SHIFT);
       right = ((xe+FIXP16_ROUND_UP)>>FIXP16_SHIFT);

       // adjust starting point and ending point
       xs+=dx_left;
       xe+=dx_right;

       // clip line
       if (left < g_clip_min_x)
          {
          left = g_clip_min_x;

          if (right < g_clip_min_x)
             continue;
          }

       if (right > g_clip_max_x)
          {
          right = g_clip_max_x;

          if (left > g_clip_max_x)
             continue;
          }

        //memset((UCHAR *)dest_addr+left, color, (right-left+1));

       } // end for

   } // end else x clipping needed

} // end GFX_Draw_Bottom_TriFP

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Fast_Distance_2D(int x, int y)
{
// this function computes the distance from 0,0 to x,y with 3.5% error

// first compute the absolute value of x,y
x = abs(x);
y = abs(y);

// compute the minimum of x,y
int mn = MIN(x,y);

// return the distance
return(x+y-(mn>>1)-(mn>>2)+(mn>>4));

} // end Fast_Distance_2D

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

float Fast_Distance_3D(float fx, float fy, float fz)
{
// this function computes the distance from the origin to x,y,z

int temp;  // used for swaping
int x,y,z; // used for algorithm

// make sure values are all positive
x = fabs(fx) * 1024;
y = fabs(fy) * 1024;
z = fabs(fz) * 1024;

// sort values
if (y < x) SWAP(x,y,temp)

if (z < y) SWAP(y,z,temp)

if (y < x) SWAP(x,y,temp)

int dist = (z + 11 * (y >> 5) + (x >> 2) );

// compute distance with 8% error
return((float)(dist >> 10));

} // end Fast_Distance_3D

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Find_Bounding_Box_Poly2D(POLYGON2D_PTR poly,
                             float *min_x, float *max_x,
                             float *min_y, float *max_y)
{
// this function finds the bounding box of a 2D polygon
// and returns the values in the sent vars

int index; // looping var

// is this poly valid?
if (poly->num_verts == 0)
    return(0);

// initialize output vars (note they are pointers)
// also note that the algorithm assumes local coordinates
// that is, the poly verts are relative to 0,0
*max_x = *max_y = *min_x = *min_y = 0;

// process each vertex
for (index=0; index < poly->num_verts; index++)
    {
    // update vars - run min/max seek
    if (poly->vlist[index].x > *max_x)
       *max_x = poly->vlist[index].x;

    if (poly->vlist[index].x < *min_x)
       *min_x = poly->vlist[index].x;

    if (poly->vlist[index].y > *max_y)
       *max_y = poly->vlist[index].y;

    if (poly->vlist[index].y < *min_y)
       *min_y = poly->vlist[index].y;

} // end for index

// return success
return(1);

} // end Find_Bounding_Box_Poly2D

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Build_Sin_Cos_Tables(void)
{
// create sin/cos lookup table
// note the creation of one extra element; 360
// this helps with logic in using the tables

int ang; 

// generate the tables 0 - 360 inclusive
for (ang = 0; ang <= 360; ang++)
    {
    // convert ang to radians
    float theta = (float)ang*PI/(float)180;

    // insert next entry into table
    cos_look[ang] = cos(theta);
    sin_look[ang] = sin(theta);

    } // end for ang

} // end Build_Sin_Cos_Tables

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Translate_Polygon2D(POLYGON2D_PTR poly, int dx, int dy)
{
// this function translates the center of a polygon

// test for valid pointer
if (!poly)
   return(0);

// translate
poly->x0+=dx;
poly->y0+=dy;

// return success
return(1);

} // end Translate_Polygon2D

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Rotate_Polygon2D(POLYGON2D_PTR poly, int theta)
{
// this function rotates the local coordinates of the polygon

int curr_vert;
float ftheta;

// test for valid pointer
if (!poly)
   return(0);

// test for negative rotation angle
if (theta < 0)
   theta+=360;

// conver to radians
ftheta = DEG_TO_RAD( theta );

// loop and rotate each point, very crude, no lookup!!!
for (curr_vert = 0; curr_vert < poly->num_verts; curr_vert++)
    {

    // perform rotation
//    float xr = (float)poly->vlist[curr_vert].x*cos_look[theta] -
//                    (float)poly->vlist[curr_vert].y*sin_look[theta];

//    float yr = (float)poly->vlist[curr_vert].x*sin_look[theta] +
//                    (float)poly->vlist[curr_vert].y*cos_look[theta];

    // later convert back to sin/cos look up table when memory is freed
    float xr = (float)poly->vlist[curr_vert].x*cos( ftheta ) -
                    (float)poly->vlist[curr_vert].y*sin( ftheta );

    float yr = (float)poly->vlist[curr_vert].x*sin( ftheta ) +
                    (float)poly->vlist[curr_vert].y*cos( ftheta );

    // store result back
    poly->vlist[curr_vert].x = xr;
    poly->vlist[curr_vert].y = yr;

    } // end for curr_vert

// return success
return(1);

} // end Rotate_Polygon2D

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Scale_Polygon2D(POLYGON2D_PTR poly, float sx, float sy)
{
// this function scalesthe local coordinates of the polygon

int curr_vert;

// test for valid pointer
if (!poly)
   return(0);

// loop and scale each point
for (curr_vert = 0; curr_vert < poly->num_verts; curr_vert++)
    {
    // scale and store result back
    poly->vlist[curr_vert].x *= sx;
    poly->vlist[curr_vert].y *= sy;

    } // end for curr_vert

// return success
return(1);

} // end Scale_Polygon2D

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Draw_Polygon2D(POLYGON2D_PTR poly, UCHAR *vbuffer)
{
// this function draws a POLYGON2D based on

int index; // looping var

// test if the polygon is visible
if (poly->state)
   {
   // loop thru and draw a line from vertices 1 to n
   for (index=0; index < poly->num_verts-1; index++)
        {
        // draw line from ith to ith+1 vertex
        GFX_Draw_Clip_Line(poly->vlist[index].x+poly->x0,
                           poly->vlist[index].y+poly->y0,
                           poly->vlist[index+1].x+poly->x0,
                           poly->vlist[index+1].y+poly->y0,
                           poly->color, vbuffer);

        } // end for

       // now close up polygon
       // draw line from last vertex to 0th
       GFX_Draw_Clip_Line(poly->vlist[0].x+poly->x0,
                          poly->vlist[0].y+poly->y0,
                          poly->vlist[index].x+poly->x0,
                          poly->vlist[index].y+poly->y0,
                          poly->color, vbuffer);

   // return success
   return(1);
   } // end if
else
   return(0);

} // end GFX_Draw_Polygon2D

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// these are the matrix versions, note they are more inefficient for
// single transforms, but their power comes into play when you concatenate
// multiple transformations, not to mention that all transforms are accomplished
// with the same code, just the matrix differs

int Translate_Polygon2D_Mat(POLYGON2D_PTR poly, int dx, int dy)
{
// this function translates the center of a polygon by using a matrix multiply
// on the the center point, this is incredibly inefficient, but for educational purposes
// if we had an object that wasn't in local coordinates then it would make more sense to
// use a matrix, but since the origin of the object is at x0,y0 then 2 lines of code can
// translate, but lets do it the hard way just to see :)

// test for valid pointer
if (!poly)
   return(0);

MATRIX3X2 mt; // used to hold translation transform matrix

// initialize the matrix with translation values dx dy
Mat_Init_3X2(&mt,1,0, 0,1, dx, dy);

// create a 1x2 matrix to do the transform
MATRIX1X2 p0;
MATRIX1X2 p1; // this will hold result

p0.M00 = poly->x0;
p0.M01 = poly->y0;

p1.M00 = 0;
p1.M01 = 0;

// now translate via a matrix multiply
Mat_Mul_1X2_3X2(&p0, &mt, &p1);

// now copy the result back into polygon
poly->x0 = p1.M[0];
poly->y0 = p1.M[1];

// return success
return(1);

} // end Translate_Polygon2D_Mat

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Rotate_Polygon2D_Mat(POLYGON2D_PTR poly, int theta)
{
// this function rotates the local coordinates of the polygon

int curr_vert;
float ftheta;

// test for valid pointer
if (!poly)
   return(0);

// test for negative rotation angle
if (theta < 0)
   theta+=360;

MATRIX3X2 mr; // used to hold rotation transform matrix

// convert angle to radians for use by math sin/cos, later go back to degrees and lookup tables
ftheta = DEG_TO_RAD(theta);

// initialize the matrix with translation values dx dy
//Mat_Init_3X2(&mr,cos_look[theta],sin_look[theta],
//                 -sin_look[theta],cos_look[theta],
//                  0, 0);

Mat_Init_3X2(&mr,cos(ftheta),sin(ftheta),
                 -sin(ftheta),cos(ftheta),
                  0, 0);

// loop and rotate each point, very crude, no lookup!!!
for (curr_vert = 0; curr_vert < poly->num_verts; curr_vert++)
    {
    // create a 1x2 matrix to do the transform
    MATRIX1X2 p0; 
    MATRIX1X2 p1; // this will hold result

    p0.M00 = poly->vlist[curr_vert].x;
    p0.M01 = poly->vlist[curr_vert].y;

    p1.M00 = 0;
    p1.M01 = 0;

    // now rotate via a matrix multiply
    Mat_Mul_1X2_3X2(&p0, &mr, &p1);

    // now copy the result back into vertex
    poly->vlist[curr_vert].x = p1.M[0];
    poly->vlist[curr_vert].y = p1.M[1];

    } // end for curr_vert

// return success
return(1);

} // end Rotate_Polygon2D_Mat

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Scale_Polygon2D_Mat(POLYGON2D_PTR poly, float sx, float sy)
{
// this function scalesthe local coordinates of the polygon
int curr_vert;

// test for valid pointer
if (!poly)
   return(0);


MATRIX3X2 ms; // used to hold scaling transform matrix

// initialize the matrix with translation values dx dy
Mat_Init_3X2(&ms,sx,0,
                 0,sy,
                 0, 0);

// loop and scale each point
for (curr_vert = 0; curr_vert < poly->num_verts; curr_vert++)
    {
    // scale and store result back

    // create a 1x2 matrix to do the transform
    MATRIX1X2 p0; 
    MATRIX1X2 p1; // this will hold result

    p0.M00 = poly->vlist[curr_vert].x;
    p0.M01 = poly->vlist[curr_vert].y;

    p1.M00 = 0;
    p1.M01 = 0;

    // now scale via a matrix multiply
    Mat_Mul_1X2_3X2(&p0, &ms, &p1);

    // now copy the result back into vertex
    poly->vlist[curr_vert].x = p1.M[0];
    poly->vlist[curr_vert].y = p1.M[1];

    } // end for curr_vert

// return success
return(1);

} // end Scale_Polygon2D_Mat


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Mat_Mul_3X3(MATRIX3X3_PTR ma,
               MATRIX3X3_PTR mb,
               MATRIX3X3_PTR mprod)
{
// this function multiplies two matrices together and
// and stores the result

int row, col, index;

for (row=0; row<3; row++)
    {
    for (col=0; col<3; col++)
        {
        // compute dot product from row of ma
        // and column of mb

        float sum = 0; // used to hold result

        for (index=0; index<3; index++)
             {
             // add in next product pair
             sum+=(ma->M[row][index]*mb->M[index][col]);
             } // end for index

        // insert resulting row,col element
        mprod->M[row][col] = sum;

        } // end for col

    } // end for row

return(1);

} // end Mat_Mul_3X3

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Mat_Mul_1X3_3X3(MATRIX1X3_PTR ma,
                   MATRIX3X3_PTR mb,
                   MATRIX1X3_PTR mprod)
{
// this function multiplies a 1x3 matrix against a
// 3x3 matrix - ma*mb and stores the result
int col, index;

    for (col=0; col<3; col++)
        {
        // compute dot product from row of ma
        // and column of mb

        float sum = 0; // used to hold result

        for (index=0; index<3; index++)
             {
             // add in next product pair
             sum+=(ma->M[index]*mb->M[index][col]);
             } // end for index

        // insert resulting col element
        mprod->M[col] = sum;

        } // end for col

return(1);

} // end Mat_Mul_1X3_3X3

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Mat_Mul_1X2_3X2(MATRIX1X2_PTR ma,
                   MATRIX3X2_PTR mb,
                   MATRIX1X2_PTR mprod)
{
// this function multiplies a 1x2 matrix against a
// 3x2 matrix - ma*mb and stores the result
// using a dummy element for the 3rd element of the 1x2
// to make the matrix multiply valid i.e. 1x3 X 3x2

int index, col; // looping var

    for (col=0; col<2; col++)
        {
        // compute dot product from row of ma
        // and column of mb

        float sum = 0; // used to hold result

        for (index=0; index<2; index++)
             {
             // add in next product pair
             sum+=(ma->M[index]*mb->M[index][col]);
             } // end for index

        // add in last element * 1
        sum+= mb->M[index][col];

        // insert resulting col element
        mprod->M[col] = sum;

        } // end for col

return(1);

} // end Mat_Mul_1X2_3X2

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Mat_Init_3X2(MATRIX3X2_PTR ma,
                 float m00, float m01,
                 float m10, float m11,
                 float m20, float m21)
{
// this function fills a 3x2 matrix with the sent data in row major form
ma->M[0][0] = m00; ma->M[0][1] = m01;
ma->M[1][0] = m10; ma->M[1][1] = m11;
ma->M[2][0] = m20; ma->M[2][1] = m21;

// return success
return(1);

} // end Mat_Init_3X2

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

FIXP16 FIXP16_DIV(FIXP16 fp1, FIXP16 fp2)
{
// this function computes the quotient fp1/fp2 using
// 32 bit math, try to keep as much precision as possible

// compute fp1/fp2, for result not to underflow the divisor needs to be shifted to the left 16 digits, however
// this would overflow the datatype, thus a comprimise must be struck where we shift the divisor legt 6 digits
// and the dividend right 10 digits, thus the resulting division will be back in 16.16 format, but the range
// of the dividend was cut to 10 digits of integer accuracy and the divisor to 6 digits of fractional accuracy

return ( (fp1 << 6) / (fp2 >> 10) );

} // end FIXP16_DIV

/////////////////////////////////////////////////////////////////////////

FIXP16 FIXP16_MUL(FIXP16 fp1, FIXP16 fp2)
{
// this function computes the product fp_prod = fp1*fp2
// using 32 bit math, try to keep as much precission as possible

// compute fp1*fp2, for result not to overflow, we must shift to the right 8 times each operand, so final product
// is stil in 16.16 format, this however, causes the loss of 8 bits of precision, the correct way to do this would
// be to use a full 64-bit multiply then shift the result back, but not worth it probably due to overhead
return ( (fp1 >> 8) * (fp2 >> 8) );

} // end FIXP16_MUL

///////////////////////////////////////////////////////////

float Fast_Sin(float theta)
{
// this function uses the sin_look[] lookup table, but
// has logic to handle negative angles as well as fractional
// angles via interpolation, use this for a more robust
// sin computation that the blind lookup, but with with
// a slight hit in speed

// convert angle to 0-359, used to use fmodf()
theta = fmod(theta,360);

// make angle positive
if (theta < 0) theta+=360.0;

// compute floor of theta and fractional part to interpolate
int theta_int    = (int)theta;
float theta_frac = theta - theta_int;

// now compute the value of sin(angle) using the lookup tables
// and interpolating the fractional part, note that if theta_int
// is equal to 359 then theta_int+1=360, but this is fine since the
// table was made with the entries 0-360 inclusive
return(sin_look[theta_int] +
       theta_frac*(sin_look[theta_int+1] - sin_look[theta_int]));

} // end Fast_Sin

/////////////////////////////////////////////////////////////////////////

float Fast_Cos(float theta)
{
// this function uses the cos_look[] lookup table, but
// has logic to handle negative angles as well as fractional
// angles via interpolation, use this for a more robust
// cos computation that the blind lookup, but with with
// a slight hit in speed

// convert angle to 0-359
theta = fmod(theta,360);

// make angle positive
if (theta < 0) theta+=360.0;

// compute floor of theta and fractional part to interpolate
int theta_int    = (int)theta;
float theta_frac = theta - theta_int;

// now compute the value of sin(angle) using the lookup tables
// and interpolating the fractional part, note that if theta_int
// is equal to 359 then theta_int+1=360, but this is fine since the
// table was made with the entries 0-360 inclusive
return(cos_look[theta_int] +
       theta_frac*(cos_look[theta_int+1] - cos_look[theta_int]));

} // end Fast_Cos

/////////////////////////////////////////////////////////////////////////

// Tile map functions, added 11/10/08

void GFX_TMap_Print_Char(char ch)
{
// this function prints a character to the next terminal position and updates the cursor position
// if the character is not  " ", "0...9", ".,", "A...Z" then the function prints a space character

// test for control code first
if (ch == 0x0A || ch == 0x0D)
    {
    // reset cursor to left side of screen
    g_tmap_cursor_x = 0;

    if (++g_tmap_cursor_y >= g_tile_map_height)
       {
       g_tmap_cursor_y = g_tile_map_height-1; 

       // scroll tile map data one line up vertically
       memcpy( tile_map_ptr, tile_map_ptr + g_tile_map_width, g_tile_map_width*(g_tile_map_height-1) );

       // clear last line out 
       memset( tile_map_ptr + g_tile_map_width*(g_tile_map_height-1), TMAP_BASE_BLANK, g_tile_map_width );
    
       } // end if

    // return to caller
    return;

    } // end if
else // normal printable character?
    {
    // convert to upper case first
    ch = toupper(ch);

    // now test if character is in range
    if (ch >= 'A' && ch <= 'Z') ch = ch - 'A' + TMAP_BASE_LETTERS;
    else
    if (ch >= '0' && ch <= '9') ch = ch - '0' + TMAP_BASE_NUMBERS;
    else
    if (ch == '.') ch = ch - '.' + TMAP_BASE_PUNC;
    else
    if (ch == ',') ch = ch - ',' + TMAP_BASE_PUNC+1;
    else 
       ch = TMAP_BASE_BLANK;

    // write character to tile_map
    tile_map_ptr[g_tmap_cursor_x + (g_tmap_cursor_y * g_tile_map_width) ] = ch;

    // update cursor position
    if (++g_tmap_cursor_x >= g_tile_map_width)
       {
       g_tmap_cursor_x = 0;

       if (++g_tmap_cursor_y >= g_tile_map_height)
          {
          g_tmap_cursor_y = g_tile_map_height-1; 

          // scroll tile map data one line up vertically
          memcpy( tile_map_ptr, tile_map_ptr + g_tile_map_width, g_tile_map_width*(g_tile_map_height-1) );

          // clear last line out 
          memset( tile_map_ptr + g_tile_map_width*(g_tile_map_height-1), TMAP_BASE_BLANK, g_tile_map_width );
          } // end if

       } // end if

    // return to caller
    return;

    } // end else

} // end GFX_TMap_Print_Char

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void GFX_TMap_Print_String(char *string)
{
// this function prints the sent string by making repeated calls to the character print function
while( *string != NULL)
   GFX_TMap_Print_Char( *string++ );

} // end GFX_TMap_Print_String

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GFX_TMap_CLS(char tile_index)
{
// this function clears the tile map memory out with the sent absolute tile index

// clear the memory
memset( tile_map_ptr, tile_index, g_tile_map_width*g_tile_map_height );

// home cursor
g_tmap_cursor_x = 0;
g_tmap_cursor_y = 0;

} // end GFX_TMap_CLS

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GFX_TMap_Setcursor(int x, int y)
{
// this function sets the cursor position for the tile mapped console

// test if cursor is on screen
if (x < 0) x = 0;
else
if (x >= g_tile_map_width ) x = g_tile_map_width - 1;

if (y < 0) y = 0;
else
if (y >= g_tile_map_height ) y = g_tile_map_height - 1;

// make assignment
g_tmap_cursor_x = x;
g_tmap_cursor_y = y;

} // end GFX_TMap_Setcursor

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
